# Check that we are using 2015.4 or later
# See Xilinx quality alert at:
# http://www.xilinx.com/support/documentation/customer_notices/xcn15040.pdf
set viv_ver [split [version -short] .]
set viv_maj [lindex $viv_ver 0]
set viv_min [lindex $viv_ver 1]

if { $viv_maj eq "2015" && $viv_min eq "4" || $viv_maj > "2015" } {
} else {
  puts "Vivado ${viv_maj}.${viv_min} is not supported for model ADM-XRC-KU1."
  puts "Please review http://www.xilinx.com/support/documentation/customer_notices/xcn15040.pdf"
  return -1
}

# Get the directory containing this script. Source file paths are relative to this.
set origin_dir [file dirname [file normalize [info script]]]

# When adapting this script for a different FPGA design or different device, it should be
# necessary to change only the following five variables.
set design {ddr4_test}
set config {8g_2400_x32_axi4}
set device {ku115_2e}
set dir "${config}-${device}"
set bit_path [file normalize "${origin_dir}/${dir}/${design}.runs/impl_1/${design}.bit"]

# The logic for this script is that we look for a .bit file in the same directory as this script; if it exists, we use it.
# Otherwise, expect the .bit file in 'bit_relpath', as set above.
set bit_filename [file tail $bit_path]
if { [file exists "${origin_dir}/${bit_filename}" ] } {
  set bit_dirpath $origin_dir
} else {
  set bit_dirpath [file dirname $bit_path]
}


open_hw

set old_hw_server [ get_hw_server ]
if { $old_hw_server ne {} } {
  disconnect_hw_server $old_hw_server
}

connect_hw_server

set target [lindex [get_hw_targets] 0]
open_hw_target $target

set device [lindex [get_hw_devices] 1]
set_property PROBES.FILE "${bit_dirpath}/debug_nets.ltx" $device
set_property PROGRAM.FILE "${bit_dirpath}/${bit_filename}" $device
program_hw_devices $device
refresh_hw_device $device

set vio_list [get_hw_vios]
foreach vio $vio_list {
  set inst_name [get_property INSTANCE_NAME $vio]
  if { $inst_name eq {gen_vio_g.vio0_i} } {
    set vio_bank0 $vio
  }
  if { $inst_name eq {gen_vio_g.vio1_i} } {
    set vio_bank1 $vio
  }
  if { $inst_name eq {gen_vio_g.vio2_i} } {
    set vio_bank2 $vio
  }
  if { $inst_name eq {gen_vio_g.vio3_i} } {
    set vio_bank3 $vio
  }
}

set custom_names_in [list \
  {init_calib_complete} \
  {ctl_done} \
  {ctl_error} \
  {ctl_efull} \
  {ctl_elost} \
  {ctl_ecount[31:0]} \
  {ctl_ephase[2:0]} \
  {ctl_eaddr[31:0]} \
  {ctl_edata[31:0]} \
  {ctl_ebits[31:0]} \
  {ctl_edata[63:32]} \
  {ctl_ebits[63:32]} \
  {ctl_edata[95:64]} \
  {ctl_ebits[95:64]} \
  {ctl_edata[127:96]} \
  {ctl_ebits[127:96]} \
  {ctl_edata[159:128]} \
  {ctl_ebits[159:128]} \
  {ctl_edata[191:160]} \
  {ctl_ebits[191:160]} \
  {ctl_edata[223:192]} \
  {ctl_ebits[223:192]} \
  {ctl_edata[255:224]} \
  {ctl_ebits[255:224]} \
]
set custom_names_out [list \
  {vio_reset} \
  {ctl_offset[31:0]} \
  {ctl_length[31:0]} \
  {ctl_eindex[7:0]} \
  {ctl_ph_en[7:0]} \
]

set vios [list $vio_bank0 $vio_bank1 $vio_bank2 $vio_bank3]
set i 0
foreach vio $vios {
  set probes_in [get_hw_probes -filter {TYPE == "vio_input"} -of_objects $vio]
  foreach probe $probes_in {
    set probe_index [get_property PROBE_PORT $probe]
    set_property NAME.CUSTOM "B$i:[lindex $custom_names_in $probe_index]" $probe
    set_property NAME.SELECT {Custom} $probe
    set_property INPUT_VALUE_RADIX {HEX} $probe
  }
  set probes_out [get_hw_probes -filter {TYPE == "vio_output"} -of_objects $vio]
  foreach probe $probes_out {
    set probe_index [get_property PROBE_PORT $probe]
    set_property NAME.CUSTOM "B$i:[lindex $custom_names_out $probe_index]" $probe
    set_property NAME.SELECT {Custom} $probe
    set_property OUTPUT_VALUE_RADIX {HEX} $probe
  }
  incr i;
}
