# Check that we are using 2015.4 or later
# See Xilinx quality alert at:
# http://www.xilinx.com/support/documentation/customer_notices/xcn15040.pdf
set viv_ver [version -short]
set viv_spl [split $viv_ver .]
set viv_maj [lindex $viv_spl 0]
set viv_min [lindex $viv_spl 1]

if { ($viv_maj == "2015" && $viv_min >= "4") || $viv_maj > "2015" } {
} else {
  puts "Vivado ${viv_ver} is not supported for model ADM-XRC-KU1."
  puts "Please review http://www.xilinx.com/support/documentation/customer_notices/xcn15040.pdf"
  return -1
}

# Get the directory containing this script. Source file paths are relative to this.
set origin_dir [file dirname [file normalize [info script]]]

# Define versions of dependencies in one convenient place
source "$origin_dir/version.tcl"

# Create project
set dir {8g_2400_x32_native-ku060_2e}
set part {xcku060-ffva1517-2-e}
set ip_part {ku060_2e}
set name {ddr4_test}
create_project -force -part $part $name "$origin_dir/$dir"

# Set the directory path for the new project
set proj_dir [get_property directory [current_project]]

# Set project properties
set project [get_projects $name]
set_property "default_lib" "xil_defaultlib" $project
set_property "part" $part $project
set_property "target_language" "VHDL" $project
set_property "simulator_language" "Mixed" $project


# Create 'sources_1' fileset (if not found)
if {[string equal [get_filesets -quiet sources_1] ""]} {
  create_fileset -srcset sources_1
}
set source_set [get_filesets sources_1]

# Set source set properties
#set_property "top" "ddr4_test" $source_set

# Add HDL source files
set hdl_files [list \
  [file normalize "$origin_dir/../../../../fpga/lib/memory_exerciser-$ver_memory_exerciser/src/rng_n1024_r32_t6_k32_s2999b26.vhd"] \
  [file normalize "$origin_dir/../../../../fpga/lib/memory_exerciser-$ver_memory_exerciser/src/memory_exerciser_rng.vhd"] \
  [file normalize "$origin_dir/../../../../fpga/lib/memory_exerciser-$ver_memory_exerciser/src/memory_exerciser_data_check.vhd"] \
  [file normalize "$origin_dir/../../../../fpga/lib/memory_exerciser-$ver_memory_exerciser/src/memory_exerciser_fifo.vhd"] \
  [file normalize "$origin_dir/../../../../fpga/lib/memory_exerciser-$ver_memory_exerciser/src/memory_exerciser_error_log.vhd"] \
  [file normalize "$origin_dir/../../../../fpga/lib/memory_exerciser-$ver_memory_exerciser/src/memory_exerciser_data_gen.vhd"] \
  [file normalize "$origin_dir/../../../../fpga/lib/memory_exerciser-$ver_memory_exerciser/src/memory_exerciser_rearranger.vhd"] \
  [file normalize "$origin_dir/../../../../fpga/lib/memory_exerciser-$ver_memory_exerciser/src/memory_exerciser_dearranger.vhd"] \
  [file normalize "$origin_dir/../../../../fpga/lib/memory_exerciser-$ver_memory_exerciser/src/memory_exerciser_mig_native.vhd"] \
  [file normalize "$origin_dir/../src/ddr4_test_native.vhd"] \
]
if { [llength $hdl_files] > 0 } {
  add_files -norecurse -fileset $source_set $hdl_files
}

# Import IP sources
if { $viv_maj >= 2016 } {
  set ip_subdir 2016.1
} else {
  set ip_subdir 2015.4
}
set ip_files [list \
  [file normalize "$origin_dir/../ip/$ip_subdir/ddr4sdram_8g_2400_x32_native-${ip_part}/ddr4sdram.xci"] \
]
if { [llength $ip_files] > 0 } {
  import_ip -srcset $source_set $ip_files
}
upgrade_ip [get_ips]

if { $viv_maj == 2015 } {
  # Set custom part for DDR4 SDRAM interface.
  # Unfortunately, .xci file stores an absolute path to .csv file containing custom parts,
  # so we cannot set this property on the .xci file that we have imported. So set it here
  # instead.
  set mig_ip [get_ips ddr4sdram]
  set_property -dict [list \
    {CONFIG.C0.DDR4_isCustom} {false} \
    {CONFIG.C0.DDR4_CustomParts} {no_file_loaded} \
  ] $mig_ip
  set_property -dict [list \
    {CONFIG.C0.DDR4_isCustom} {true} \
    {CONFIG.C0.DDR4_CustomParts} [file normalize "$origin_dir/../ip/$ip_subdir/custom_parts.csv"] \
    {CONFIG.C0.DDR4_MemoryPart} {CUSTOM_MT40A512M16HA-083E} \
  ] $mig_ip
  set_property {CONFIG.Simulation_Mode} {Unisim} $mig_ip
}
#report_property -all $mig_ip


# Create IP
create_ip -vlnv "xilinx.com:ip:vio:*" -module_name vio_ip
set vio_ip [get_ips vio_ip]

# Set initial value of probe_out2 (memory exerciser number of 32-byte words) to 0x03FFFFFF, for 2x1 GiB parts per bank
set_property -dict { \
  CONFIG.C_NUM_PROBE_IN {24} \
  CONFIG.C_NUM_PROBE_OUT {5} \
  CONFIG.C_PROBE_IN0_WIDTH {1} \
  CONFIG.C_PROBE_IN1_WIDTH {1} \
  CONFIG.C_PROBE_IN2_WIDTH {1} \
  CONFIG.C_PROBE_IN3_WIDTH {1} \
  CONFIG.C_PROBE_IN4_WIDTH {1} \
  CONFIG.C_PROBE_IN5_WIDTH {32} \
  CONFIG.C_PROBE_IN6_WIDTH {3} \
  CONFIG.C_PROBE_IN7_WIDTH {32} \
  CONFIG.C_PROBE_IN8_WIDTH {32} \
  CONFIG.C_PROBE_IN9_WIDTH {32} \
  CONFIG.C_PROBE_IN10_WIDTH {32} \
  CONFIG.C_PROBE_IN11_WIDTH {32} \
  CONFIG.C_PROBE_IN12_WIDTH {32} \
  CONFIG.C_PROBE_IN13_WIDTH {32} \
  CONFIG.C_PROBE_IN14_WIDTH {32} \
  CONFIG.C_PROBE_IN15_WIDTH {32} \
  CONFIG.C_PROBE_IN16_WIDTH {32} \
  CONFIG.C_PROBE_IN17_WIDTH {32} \
  CONFIG.C_PROBE_IN18_WIDTH {32} \
  CONFIG.C_PROBE_IN19_WIDTH {32} \
  CONFIG.C_PROBE_IN20_WIDTH {32} \
  CONFIG.C_PROBE_IN21_WIDTH {32} \
  CONFIG.C_PROBE_IN22_WIDTH {32} \
  CONFIG.C_PROBE_IN23_WIDTH {32} \
  CONFIG.C_PROBE_OUT0_WIDTH {1} \
  CONFIG.C_PROBE_OUT1_WIDTH {32} \
  CONFIG.C_PROBE_OUT2_WIDTH {32} \
  CONFIG.C_PROBE_OUT3_WIDTH {8} \
  CONFIG.C_PROBE_OUT4_WIDTH {8} \
  CONFIG.C_PROBE_OUT0_INIT_VAL {0x0} \
  CONFIG.C_PROBE_OUT1_INIT_VAL {0x00000000} \
  CONFIG.C_PROBE_OUT2_INIT_VAL {0x03FFFFFF} \
  CONFIG.C_PROBE_OUT3_INIT_VAL {0x00} \
  CONFIG.C_PROBE_OUT4_INIT_VAL {0xFF} \
} $vio_ip


# Create 'constrs_1' fileset (if not found)
if {[string equal [get_filesets -quiet constrs_1] ""]} {
  create_fileset -constrset constrs_1
}
set constraint_set [get_filesets constrs_1]

# Add constraints files
# Put target .xdc as first in list
set constraint_files [list \
  [file normalize "$origin_dir/../src/refclk300.xdc"] \
  [file normalize "$origin_dir/../src/ddr4sdram.xdc"] \
  [file normalize "$origin_dir/../src/ddr4sdram_locs_b0_x32.xdc"] \
  [file normalize "$origin_dir/../src/ddr4sdram_locs_b1_x32.xdc"] \
  [file normalize "$origin_dir/../src/ddr4sdram_locs_b2_x32.xdc"] \
  [file normalize "$origin_dir/../src/ddr4sdram_locs_b3_x32.xdc"] \
  [file normalize "$origin_dir/../src/bitstream.xdc"] \
  [file normalize "$origin_dir/../src/ddr4_test_native.xdc"] \
]
add_files -norecurse -fileset $constraint_set $constraint_files
set_property "target_constrs_file" [lindex $constraint_files end] $constraint_set


# Create 'sim_mig' fileset (if not found)
if {[string equal [get_filesets -quiet sim_mig] ""]} {
  create_fileset -simset sim_mig
}
set sim_set [get_filesets sim_mig]

# Add simulation HDL source files
set sim_files [list \
  [file normalize "$origin_dir/../../../../fpga/lib/ddr4_sdram_model-$ver_ddr4_sdram_model/src/ddr4_sdram_model_pkg.vhd"] \
  [file normalize "$origin_dir/../../../../fpga/lib/ddr4_sdram_model-$ver_ddr4_sdram_model/src/ddr4_sdram_model_int_pkg.vhd"] \
  [file normalize "$origin_dir/../../../../fpga/lib/ddr4_sdram_model-$ver_ddr4_sdram_model/src/ddr4_sdram_bank_state.vhd"] \
  [file normalize "$origin_dir/../../../../fpga/lib/ddr4_sdram_model-$ver_ddr4_sdram_model/src/ddr4_sdram_data.vhd"] \
  [file normalize "$origin_dir/../../../../fpga/lib/ddr4_sdram_model-$ver_ddr4_sdram_model/src/ddr4_sdram_init_monitor.vhd"] \
  [file normalize "$origin_dir/../../../../fpga/lib/ddr4_sdram_model-$ver_ddr4_sdram_model/src/ddr4_sdram_power_state.vhd"] \
  [file normalize "$origin_dir/../../../../fpga/lib/ddr4_sdram_model-$ver_ddr4_sdram_model/src/ddr4_sdram_refresh_monitor.vhd"] \
  [file normalize "$origin_dir/../../../../fpga/lib/ddr4_sdram_model-$ver_ddr4_sdram_model/src/ddr4_sdram_su_h_check.vhd"] \
  [file normalize "$origin_dir/../../../../fpga/lib/ddr4_sdram_model-$ver_ddr4_sdram_model/src/ddr4_sdram_die.vhd"] \
  [file normalize "$origin_dir/../../../../fpga/lib/ddr4_sdram_model-$ver_ddr4_sdram_model/src/ddr4_sdram_chip.vhd"] \
  [file normalize "$origin_dir/../../../../fpga/lib/ddr4_sdram_model-$ver_ddr4_sdram_model/src/ddr4_sdram_mt40a_pkg.vhd"] \
  [file normalize "$origin_dir/../src/tb_ddr4_test.vhd"] \
]
add_files -norecurse -fileset $sim_set $sim_files

# Set fileset properties
set_property "source_set" "sources_1" $sim_set
set_property "top" "tb_ddr4_test" $sim_set

set_property "target_simulator" "XSim" $project
set_property "xsim.simulate.runtime" "25 us" $sim_set
set_property "xsim.tclbatch" [file normalize "$origin_dir/xsim_wave_native.tcl"] $sim_set

set_property "target_simulator" "ModelSim" $project
set_property "modelsim.simulate.runtime" "25 us" $sim_set
set_property "modelsim.simulate.custom_udo" [file normalize "$origin_dir/vsim_wave_native.udo"] $sim_set

update_compile_order -fileset $sim_set

# Make sim_mig the active simset
current_fileset -simset $sim_set

if {![string equal [get_filesets -quiet sim_1] ""]} {
  delete_fileset sim_1
}


# Create 'synth_1' run (if not found)
if {[string equal [get_runs -quiet synth_1] ""]} {
  create_run -name synth_1 -part $part -flow {Vivado Synthesis 2015} -strategy "Vivado Synthesis Defaults" -constrset constrs_1
} else {
  set_property strategy "Vivado Synthesis Defaults" [get_runs synth_1]
  set_property flow "Vivado Synthesis 2015" [get_runs synth_1]
}
set synth_run [get_runs synth_1]
set_property strategy {Flow_PerfOptimized_high} $synth_run
set_property "needs_refresh" "1" $synth_run
set_property "part" $part $synth_run
current_run -synthesis $synth_run

# Create 'impl_1' run (if not found)
if {[string equal [get_runs -quiet impl_1] ""]} {
  create_run -name impl_1 -part $part -flow {Vivado Implementation 2015} -strategy "Vivado Implementation Defaults" -constrset constrs_1 -parent_run synth_1
} else {
  set_property strategy "Vivado Implementation Defaults" [get_runs impl_1]
  set_property flow "Vivado Implementation 2015" [get_runs impl_1]
}
set impl_run [get_runs impl_1]
set_property "needs_refresh" "1" $impl_run
set_property "part" $part $impl_run

set_property strategy {Performance_ExplorePostRoutePhysOpt} $impl_run
set_property -name {steps.route_design.args.more options} -value -timing_summary -objects $impl_run
current_run -implementation $impl_run

close_project

puts "INFO: Created project '$dir/$name'"
