library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_misc.all;
use ieee.numeric_std.all;

library work;
use work.ddr4_sdram_model.all;
use work.ddr4_sdram_mt40a.all;

entity tb_ddr4_test is
end entity;

architecture behav of tb_ddr4_test is

  constant MODEL_NAME : string(1 to 11) := "ADM-XRC-KU1";

  -- Clocks
  constant REFCLK300_PERIOD : time := 3.333 ns;
  constant SYS_CLK_PERIOD   : time := 3.333 ns;

  signal refclk300_p : std_logic := '0';
  signal refclk300_n : std_logic := '1';

  signal stat_rst  : std_logic_vector(3 downto 0);
  signal stat_err  : std_logic_vector(3 downto 0);
  signal stat_done : std_logic_vector(3 downto 0);
  signal stat_init : std_logic_vector(3 downto 0);

  signal c01_sys_clk_p : std_logic := '0';
  signal c01_sys_clk_n : std_logic := '1';
  signal c23_sys_clk_p : std_logic := '0';
  signal c23_sys_clk_n : std_logic := '1';

  signal c0_ddr4_par      : std_logic;
  signal c0_ddr4_ten      : std_logic;
  signal c0_ddr4_alert_n  : std_logic;
  signal c0_ddr4_act_n    : std_logic;
  signal c0_ddr4_adr      : std_logic_vector (16 downto 0);
  signal c0_ddr4_ba       : std_logic_vector (1 downto 0);
  signal c0_ddr4_bg       : std_logic_vector (0 to 0);
  signal c0_ddr4_cke      : std_logic_vector (0 to 0);
  signal c0_ddr4_odt      : std_logic_vector (0 to 0);
  signal c0_ddr4_cs_n     : std_logic_vector (0 to 0);
  signal c0_ddr4_ck_t     : std_logic_vector (0 to 0);
  signal c0_ddr4_ck_c     : std_logic_vector (0 to 0);
  signal c0_ddr4_reset_n  : std_logic;
  signal c0_ddr4_dm_dbi_n : std_logic_vector (3 downto 0);
  signal c0_ddr4_dq       : std_logic_vector (31 downto 0);
  signal c0_ddr4_dqs_t    : std_logic_vector (3 downto 0);
  signal c0_ddr4_dqs_c    : std_logic_vector (3 downto 0);

  signal c1_ddr4_par      : std_logic;
  signal c1_ddr4_ten      : std_logic;
  signal c1_ddr4_alert_n  : std_logic;
  signal c1_ddr4_act_n    : std_logic;
  signal c1_ddr4_adr      : std_logic_vector (16 downto 0);
  signal c1_ddr4_ba       : std_logic_vector (1 downto 0);
  signal c1_ddr4_bg       : std_logic_vector (0 to 0);
  signal c1_ddr4_cke      : std_logic_vector (0 to 0);
  signal c1_ddr4_odt      : std_logic_vector (0 to 0);
  signal c1_ddr4_cs_n     : std_logic_vector (0 to 0);
  signal c1_ddr4_ck_t     : std_logic_vector (0 to 0);
  signal c1_ddr4_ck_c     : std_logic_vector (0 to 0);
  signal c1_ddr4_reset_n  : std_logic;
  signal c1_ddr4_dm_dbi_n : std_logic_vector (3 downto 0);
  signal c1_ddr4_dq       : std_logic_vector (31 downto 0);
  signal c1_ddr4_dqs_t    : std_logic_vector (3 downto 0);
  signal c1_ddr4_dqs_c    : std_logic_vector (3 downto 0);

  signal c2_ddr4_par      : std_logic;
  signal c2_ddr4_ten      : std_logic;
  signal c2_ddr4_alert_n  : std_logic;
  signal c2_ddr4_act_n    : std_logic;
  signal c2_ddr4_adr      : std_logic_vector (16 downto 0);
  signal c2_ddr4_ba       : std_logic_vector (1 downto 0);
  signal c2_ddr4_bg       : std_logic_vector (0 to 0);
  signal c2_ddr4_cke      : std_logic_vector (0 to 0);
  signal c2_ddr4_odt      : std_logic_vector (0 to 0);
  signal c2_ddr4_cs_n     : std_logic_vector (0 to 0);
  signal c2_ddr4_ck_t     : std_logic_vector (0 to 0);
  signal c2_ddr4_ck_c     : std_logic_vector (0 to 0);
  signal c2_ddr4_reset_n  : std_logic;
  signal c2_ddr4_dm_dbi_n : std_logic_vector (3 downto 0);
  signal c2_ddr4_dq       : std_logic_vector (31 downto 0);
  signal c2_ddr4_dqs_t    : std_logic_vector (3 downto 0);
  signal c2_ddr4_dqs_c    : std_logic_vector (3 downto 0);

  signal c3_ddr4_par      : std_logic;
  signal c3_ddr4_ten      : std_logic;
  signal c3_ddr4_alert_n  : std_logic;
  signal c3_ddr4_act_n    : std_logic;
  signal c3_ddr4_adr      : std_logic_vector (16 downto 0);
  signal c3_ddr4_ba       : std_logic_vector (1 downto 0);
  signal c3_ddr4_bg       : std_logic_vector (0 to 0);
  signal c3_ddr4_cke      : std_logic_vector (0 to 0);
  signal c3_ddr4_odt      : std_logic_vector (0 to 0);
  signal c3_ddr4_cs_n     : std_logic_vector (0 to 0);
  signal c3_ddr4_ck_t     : std_logic_vector (0 to 0);
  signal c3_ddr4_ck_c     : std_logic_vector (0 to 0);
  signal c3_ddr4_reset_n  : std_logic;
  signal c3_ddr4_dm_dbi_n : std_logic_vector (3 downto 0);
  signal c3_ddr4_dq       : std_logic_vector (31 downto 0);
  signal c3_ddr4_dqs_t    : std_logic_vector (3 downto 0);
  signal c3_ddr4_dqs_c    : std_logic_vector (3 downto 0);

  constant completion_timeout_period : time := 25 us;

  signal test_completed   : boolean := false;

  component ddr4_test
    generic(
      simulation         : boolean := false;
      -- Following two generics ignored if not simulation
      simulation_offset  : natural := 99;   -- Value for ctl_offset
      simulation_length  : natural := 213   -- ctl_length is set to (simulation_length - 1)
    );
    port(
      refclk300_p        : in    std_logic;
      refclk300_n        : in    std_logic;
      --
      -- Status outputs used in simulation only.
      stat_rst           : out   std_logic_vector(3 downto 0);
      stat_err           : out   std_logic_vector(3 downto 0);
      stat_done          : out   std_logic_vector(3 downto 0);
      stat_init          : out   std_logic_vector(3 downto 0);
      --
      c01_sys_clk_p      : in    std_logic;
      c01_sys_clk_n      : in    std_logic;
      c23_sys_clk_p      : in    std_logic;
      c23_sys_clk_n      : in    std_logic;
      --
      c0_ddr4_par        : out   std_logic;
      c0_ddr4_ten        : out   std_logic;
      c0_ddr4_alert_n    : in    std_logic;
      c0_ddr4_act_n      : out   std_logic;
      c0_ddr4_adr        : out   std_logic_vector (16 downto 0);
      c0_ddr4_ba         : out   std_logic_vector (1 downto 0);
      c0_ddr4_bg         : out   std_logic_vector (0 to 0);
      c0_ddr4_cke        : out   std_logic_vector (0 to 0);
      c0_ddr4_odt        : out   std_logic_vector (0 to 0);
      c0_ddr4_cs_n       : out   std_logic_vector (0 to 0);
      c0_ddr4_ck_t       : out   std_logic_vector (0 to 0);
      c0_ddr4_ck_c       : out   std_logic_vector (0 to 0);
      c0_ddr4_reset_n    : out   std_logic;
      c0_ddr4_dm_dbi_n   : inout std_logic_vector (3 downto 0);
      c0_ddr4_dq         : inout std_logic_vector (31 downto 0);
      c0_ddr4_dqs_t      : inout std_logic_vector (3 downto 0);
      c0_ddr4_dqs_c      : inout std_logic_vector (3 downto 0);
      --
      c1_ddr4_par        : out   std_logic;
      c1_ddr4_ten        : out   std_logic;
      c1_ddr4_alert_n    : in    std_logic;
      c1_ddr4_act_n      : out   std_logic;
      c1_ddr4_adr        : out   std_logic_vector (16 downto 0);
      c1_ddr4_ba         : out   std_logic_vector (1 downto 0);
      c1_ddr4_bg         : out   std_logic_vector (0 to 0);
      c1_ddr4_cke        : out   std_logic_vector (0 to 0);
      c1_ddr4_odt        : out   std_logic_vector (0 to 0);
      c1_ddr4_cs_n       : out   std_logic_vector (0 to 0);
      c1_ddr4_ck_t       : out   std_logic_vector (0 to 0);
      c1_ddr4_ck_c       : out   std_logic_vector (0 to 0);
      c1_ddr4_reset_n    : out   std_logic;
      c1_ddr4_dm_dbi_n   : inout std_logic_vector (3 downto 0);
      c1_ddr4_dq         : inout std_logic_vector (31 downto 0);
      c1_ddr4_dqs_t      : inout std_logic_vector (3 downto 0);
      c1_ddr4_dqs_c      : inout std_logic_vector (3 downto 0);
      --
      c2_ddr4_par        : out   std_logic;
      c2_ddr4_ten        : out   std_logic;
      c2_ddr4_alert_n    : in    std_logic;
      c2_ddr4_act_n      : out   std_logic;
      c2_ddr4_adr        : out   std_logic_vector (16 downto 0);
      c2_ddr4_ba         : out   std_logic_vector (1 downto 0);
      c2_ddr4_bg         : out   std_logic_vector (0 to 0);
      c2_ddr4_cke        : out   std_logic_vector (0 to 0);
      c2_ddr4_odt        : out   std_logic_vector (0 to 0);
      c2_ddr4_cs_n       : out   std_logic_vector (0 to 0);
      c2_ddr4_ck_t       : out   std_logic_vector (0 to 0);
      c2_ddr4_ck_c       : out   std_logic_vector (0 to 0);
      c2_ddr4_reset_n    : out   std_logic;
      c2_ddr4_dm_dbi_n   : inout std_logic_vector (3 downto 0);
      c2_ddr4_dq         : inout std_logic_vector (31 downto 0);
      c2_ddr4_dqs_t      : inout std_logic_vector (3 downto 0);
      c2_ddr4_dqs_c      : inout std_logic_vector (3 downto 0);
      --
      c3_ddr4_par        : out   std_logic;
      c3_ddr4_ten        : out   std_logic;
      c3_ddr4_alert_n    : in    std_logic;
      c3_ddr4_act_n      : out   std_logic;
      c3_ddr4_adr        : out   std_logic_vector (16 downto 0);
      c3_ddr4_ba         : out   std_logic_vector (1 downto 0);
      c3_ddr4_bg         : out   std_logic_vector (0 to 0);
      c3_ddr4_cke        : out   std_logic_vector (0 to 0);
      c3_ddr4_odt        : out   std_logic_vector (0 to 0);
      c3_ddr4_cs_n       : out   std_logic_vector (0 to 0);
      c3_ddr4_ck_t       : out   std_logic_vector (0 to 0);
      c3_ddr4_ck_c       : out   std_logic_vector (0 to 0);
      c3_ddr4_reset_n    : out   std_logic;
      c3_ddr4_dm_dbi_n   : inout std_logic_vector (3 downto 0);
      c3_ddr4_dq         : inout std_logic_vector (31 downto 0);
      c3_ddr4_dqs_t      : inout std_logic_vector (3 downto 0);
      c3_ddr4_dqs_c      : inout std_logic_vector (3 downto 0)
    );
  end component;

begin

  refclk300_p <= transport not refclk300_p after REFCLK300_PERIOD / 2;
  refclk300_n <= not refclk300_p;

  c01_sys_clk_p <= transport not c01_sys_clk_p after SYS_CLK_PERIOD / 2;
  c01_sys_clk_n <= not c01_sys_clk_p;

  c23_sys_clk_p <= transport not c23_sys_clk_p after SYS_CLK_PERIOD / 2;
  c23_sys_clk_n <= not c23_sys_clk_p;


  -- For 4Gib parts, each bank of DDR4 = 2 x 4Gib =  8Gib / 256 = 0x2000000 (33554432) simulation length (words)
  -- For 8Gib parts, each bank of DDR4 = 2 x 8Gib = 16Gib / 256 = 0x4000000 (67108864) simulation length (words)

  uut : ddr4_test
  generic map(
    simulation        => true             -- in    boolean := false
  )
  port map(
    refclk300_p      => refclk300_p,      -- in    std_logic
    refclk300_n      => refclk300_n,      -- in    std_logic
    --
    -- Status outputs used in simulation only.
    stat_rst         => stat_rst,         -- out   std_logic_vector(3 downto 0)
    stat_err         => stat_err,         -- out   std_logic_vector(3 downto 0)
    stat_done        => stat_done,        -- out   std_logic_vector(3 downto 0)
    stat_init        => stat_init,        -- out   std_logic_vector(3 downto 0)
    --
    c01_sys_clk_p    => c01_sys_clk_p,    -- in    std_logic
    c01_sys_clk_n    => c01_sys_clk_n,    -- in    std_logic
    c23_sys_clk_p    => c23_sys_clk_p,    -- in    std_logic
    c23_sys_clk_n    => c23_sys_clk_n,    -- in    std_logic
    --
    c0_ddr4_par      => c0_ddr4_par,      -- out   std_logic
    c0_ddr4_ten      => c0_ddr4_ten,      -- out   std_logic
    c0_ddr4_alert_n  => c0_ddr4_alert_n,  -- in    std_logic
    c0_ddr4_act_n    => c0_ddr4_act_n,    -- out   std_logic
    c0_ddr4_adr      => c0_ddr4_adr,      -- out   std_logic_vector (16 downto 0)
    c0_ddr4_ba       => c0_ddr4_ba,       -- out   std_logic_vector (1 downto 0)
    c0_ddr4_bg       => c0_ddr4_bg,       -- out   std_logic_vector (0 to 0)
    c0_ddr4_cke      => c0_ddr4_cke,      -- out   std_logic_vector (0 to 0)
    c0_ddr4_odt      => c0_ddr4_odt,      -- out   std_logic_vector (0 to 0)
    c0_ddr4_cs_n     => c0_ddr4_cs_n,     -- out   std_logic_vector (0 to 0)
    c0_ddr4_ck_t     => c0_ddr4_ck_t,     -- out   std_logic_vector (0 to 0)
    c0_ddr4_ck_c     => c0_ddr4_ck_c,     -- out   std_logic_vector (0 to 0)
    c0_ddr4_reset_n  => c0_ddr4_reset_n,  -- out   std_logic
    c0_ddr4_dm_dbi_n => c0_ddr4_dm_dbi_n, -- inout std_logic_vector (3 downto 0)
    c0_ddr4_dq       => c0_ddr4_dq,       -- inout std_logic_vector (31 downto 0)
    c0_ddr4_dqs_t    => c0_ddr4_dqs_t,    -- inout std_logic_vector (3 downto 0)
    c0_ddr4_dqs_c    => c0_ddr4_dqs_c,    -- inout std_logic_vector (3 downto 0)
    --
    c1_ddr4_par      => c1_ddr4_par,      -- out   std_logic
    c1_ddr4_ten      => c1_ddr4_ten,      -- out   std_logic
    c1_ddr4_alert_n  => c1_ddr4_alert_n,  -- in    std_logic
    c1_ddr4_act_n    => c1_ddr4_act_n,    -- out   std_logic
    c1_ddr4_adr      => c1_ddr4_adr,      -- out   std_logic_vector (16 downto 0)
    c1_ddr4_ba       => c1_ddr4_ba,       -- out   std_logic_vector (1 downto 0)
    c1_ddr4_bg       => c1_ddr4_bg,       -- out   std_logic_vector (0 to 0)
    c1_ddr4_cke      => c1_ddr4_cke,      -- out   std_logic_vector (0 to 0)
    c1_ddr4_odt      => c1_ddr4_odt,      -- out   std_logic_vector (0 to 0)
    c1_ddr4_cs_n     => c1_ddr4_cs_n,     -- out   std_logic_vector (0 to 0)
    c1_ddr4_ck_t     => c1_ddr4_ck_t,     -- out   std_logic_vector (0 to 0)
    c1_ddr4_ck_c     => c1_ddr4_ck_c,     -- out   std_logic_vector (0 to 0)
    c1_ddr4_reset_n  => c1_ddr4_reset_n,  -- out   std_logic
    c1_ddr4_dm_dbi_n => c1_ddr4_dm_dbi_n, -- inout std_logic_vector (3 downto 0)
    c1_ddr4_dq       => c1_ddr4_dq,       -- inout std_logic_vector (31 downto 0)
    c1_ddr4_dqs_t    => c1_ddr4_dqs_t,    -- inout std_logic_vector (3 downto 0)
    c1_ddr4_dqs_c    => c1_ddr4_dqs_c,    -- inout std_logic_vector (3 downto 0)
    --
    c2_ddr4_par      => c2_ddr4_par,      -- out   std_logic
    c2_ddr4_ten      => c2_ddr4_ten,      -- out   std_logic
    c2_ddr4_alert_n  => c2_ddr4_alert_n,  -- in    std_logic
    c2_ddr4_act_n    => c2_ddr4_act_n,    -- out   std_logic
    c2_ddr4_adr      => c2_ddr4_adr,      -- out   std_logic_vector (16 downto 0)
    c2_ddr4_ba       => c2_ddr4_ba,       -- out   std_logic_vector (1 downto 0)
    c2_ddr4_bg       => c2_ddr4_bg,       -- out   std_logic_vector (0 to 0)
    c2_ddr4_cke      => c2_ddr4_cke,      -- out   std_logic_vector (0 to 0)
    c2_ddr4_odt      => c2_ddr4_odt,      -- out   std_logic_vector (0 to 0)
    c2_ddr4_cs_n     => c2_ddr4_cs_n,     -- out   std_logic_vector (0 to 0)
    c2_ddr4_ck_t     => c2_ddr4_ck_t,     -- out   std_logic_vector (0 to 0)
    c2_ddr4_ck_c     => c2_ddr4_ck_c,     -- out   std_logic_vector (0 to 0)
    c2_ddr4_reset_n  => c2_ddr4_reset_n,  -- out   std_logic
    c2_ddr4_dm_dbi_n => c2_ddr4_dm_dbi_n, -- inout std_logic_vector (3 downto 0)
    c2_ddr4_dq       => c2_ddr4_dq,       -- inout std_logic_vector (31 downto 0)
    c2_ddr4_dqs_t    => c2_ddr4_dqs_t,    -- inout std_logic_vector (3 downto 0)
    c2_ddr4_dqs_c    => c2_ddr4_dqs_c,    -- inout std_logic_vector (3 downto 0)
    --
    c3_ddr4_par      => c3_ddr4_par,      -- out   std_logic
    c3_ddr4_ten      => c3_ddr4_ten,      -- out   std_logic
    c3_ddr4_alert_n  => c3_ddr4_alert_n,  -- in    std_logic
    c3_ddr4_act_n    => c3_ddr4_act_n,    -- out   std_logic
    c3_ddr4_adr      => c3_ddr4_adr,      -- out   std_logic_vector (16 downto 0)
    c3_ddr4_ba       => c3_ddr4_ba,       -- out   std_logic_vector (1 downto 0)
    c3_ddr4_bg       => c3_ddr4_bg,       -- out   std_logic_vector (0 to 0)
    c3_ddr4_cke      => c3_ddr4_cke,      -- out   std_logic_vector (0 to 0)
    c3_ddr4_odt      => c3_ddr4_odt,      -- out   std_logic_vector (0 to 0)
    c3_ddr4_cs_n     => c3_ddr4_cs_n,     -- out   std_logic_vector (0 to 0)
    c3_ddr4_ck_t     => c3_ddr4_ck_t,     -- out   std_logic_vector (0 to 0)
    c3_ddr4_ck_c     => c3_ddr4_ck_c,     -- out   std_logic_vector (0 to 0)
    c3_ddr4_reset_n  => c3_ddr4_reset_n,  -- out   std_logic
    c3_ddr4_dm_dbi_n => c3_ddr4_dm_dbi_n, -- inout std_logic_vector (3 downto 0)
    c3_ddr4_dq       => c3_ddr4_dq,       -- inout std_logic_vector (31 downto 0)
    c3_ddr4_dqs_t    => c3_ddr4_dqs_t,    -- inout std_logic_vector (3 downto 0)
    c3_ddr4_dqs_c    => c3_ddr4_dqs_c     -- inout std_logic_vector (3 downto 0)
  );


  -- Bank 0 DDR4 simulation model
  gen_chips_bank0 : for chip in 0 to 1 generate
  
    U0 : ddr4_sdram_chip
    generic map(
      model_name => "bank0_chip" & natural'image(chip),
      part => MT40A512M16_083E)
    port map(
      reset_n => c0_ddr4_reset_n,
      ck_t => c0_ddr4_ck_t(0),
      ck_c => c0_ddr4_ck_c(0),
      ten => c0_ddr4_ten,
      cke => c0_ddr4_cke(0),
      cs_n => c0_ddr4_cs_n(0),
      act_n => c0_ddr4_act_n,
      odt => c0_ddr4_odt(0),
      a => c0_ddr4_adr,
      ba => c0_ddr4_ba,
      bg(0) => c0_ddr4_bg(0),
      par => c0_ddr4_par,
      dm_dbi_n => c0_ddr4_dm_dbi_n(2 * (chip + 1) - 1 downto 2 * chip),
      dq => c0_ddr4_dq(16 * (chip + 1) - 1 downto 16 * chip),
      dqs_t => c0_ddr4_dqs_t(2 * (chip + 1) - 1 downto 2 * chip),
      dqs_c => c0_ddr4_dqs_c(2 * (chip + 1) - 1 downto 2 * chip),
      alert_n => c0_ddr4_alert_n);
  
  end generate;
    
  -- Bank 1 DDR4 simulation model
  gen_chips_bank1 : for chip in 0 to 1 generate
  
    U0 : ddr4_sdram_chip
    generic map(
      model_name => "bank1_chip" & natural'image(chip),
      part => MT40A512M16_083E)
    port map(
      reset_n => c1_ddr4_reset_n,
      ck_t => c1_ddr4_ck_t(0),
      ck_c => c1_ddr4_ck_c(0),
      ten => c1_ddr4_ten,
      cke => c1_ddr4_cke(0),
      cs_n => c1_ddr4_cs_n(0),
      act_n => c1_ddr4_act_n,
      odt => c1_ddr4_odt(0),
      a => c1_ddr4_adr,
      ba => c1_ddr4_ba,
      bg(0) => c1_ddr4_bg(0),
      par => c1_ddr4_par,
      dm_dbi_n => c1_ddr4_dm_dbi_n(2 * (chip + 1) - 1 downto 2 * chip),
      dq => c1_ddr4_dq(16 * (chip + 1) - 1 downto 16 * chip),
      dqs_t => c1_ddr4_dqs_t(2 * (chip + 1) - 1 downto 2 * chip),
      dqs_c => c1_ddr4_dqs_c(2 * (chip + 1) - 1 downto 2 * chip),
      alert_n => c1_ddr4_alert_n);
  
  end generate;
    
  -- Bank 2 DDR4 simulation model
  gen_chips_bank2 : for chip in 0 to 1 generate
  
    U0 : ddr4_sdram_chip
    generic map(
      model_name => "bank2_chip" & natural'image(chip),
      part => MT40A512M16_083E)
    port map(
      reset_n => c2_ddr4_reset_n,
      ck_t => c2_ddr4_ck_t(0),
      ck_c => c2_ddr4_ck_c(0),
      ten => c2_ddr4_ten,
      cke => c2_ddr4_cke(0),
      cs_n => c2_ddr4_cs_n(0),
      act_n => c2_ddr4_act_n,
      odt => c2_ddr4_odt(0),
      a => c2_ddr4_adr,
      ba => c2_ddr4_ba,
      bg(0) => c2_ddr4_bg(0),
      par => c2_ddr4_par,
      dm_dbi_n => c2_ddr4_dm_dbi_n(2 * (chip + 1) - 1 downto 2 * chip),
      dq => c2_ddr4_dq(16 * (chip + 1) - 1 downto 16 * chip),
      dqs_t => c2_ddr4_dqs_t(2 * (chip + 1) - 1 downto 2 * chip),
      dqs_c => c2_ddr4_dqs_c(2 * (chip + 1) - 1 downto 2 * chip),
      alert_n => c2_ddr4_alert_n);
  
  end generate;
    
  -- Bank 3 DDR4 simulation model
  gen_chips_bank3 : for chip in 0 to 1 generate
  
    U0 : ddr4_sdram_chip
    generic map(
      model_name => "bank3_chip" & natural'image(chip),
      part => MT40A512M16_083E)
    port map(
      reset_n => c3_ddr4_reset_n,
      ck_t => c3_ddr4_ck_t(0),
      ck_c => c3_ddr4_ck_c(0),
      ten => c3_ddr4_ten,
      cke => c3_ddr4_cke(0),
      cs_n => c3_ddr4_cs_n(0),
      act_n => c3_ddr4_act_n,
      odt => c3_ddr4_odt(0),
      a => c3_ddr4_adr,
      ba => c3_ddr4_ba,
      bg(0) => c3_ddr4_bg(0),
      par => c3_ddr4_par,
      dm_dbi_n => c3_ddr4_dm_dbi_n(2 * (chip + 1) - 1 downto 2 * chip),
      dq => c3_ddr4_dq(16 * (chip + 1) - 1 downto 16 * chip),
      dqs_t => c3_ddr4_dqs_t(2 * (chip + 1) - 1 downto 2 * chip),
      dqs_c => c3_ddr4_dqs_c(2 * (chip + 1) - 1 downto 2 * chip),
      alert_n => c3_ddr4_alert_n);
      
  end generate;
    

  wait_complete_p : process
  begin
    -- Wait for reset to be deasserted.
    if stat_rst /= "0000" then
      wait until stat_rst = "0000";
    end if;
    report "Training started for all banks" severity note;
    wait for 1 us;

    -- Wait for MIG training to finish.
    if stat_init /= "1111" then
      wait until stat_init = "1111";
    end if;
    report "Training completed for all banks" severity note;
    wait for 1 us;

    -- Wait for memory test to finish.
    if stat_done /= "1111" then
      wait until stat_done = "1111";
    end if;
    report "Memory test completed for all banks" severity note;
    wait for 1 us;

    test_completed <= true;

    -- Check result of memory test.
    if stat_err = "0000" then
      report "Simulation completed: PASSED" severity failure;
    else
      for b in 0 to 3 loop
        if stat_err(b) = '1' then
          report "Memory test error for bank " & integer'image(b) severity error;
        end if;
      end loop;
      report "*** Simulation completed: FAILED" severity failure;
    end if;
  end process;

  check_for_timeout_p : process
  begin
    wait for completion_timeout_period;
    if not test_completed then
      for b in 0 to 3 loop
        if stat_rst(b) = '1' then
          report "Reset not completed for bank " & integer'image(b) severity error;
        elsif stat_init(b) = '0' then
          report "Training not completed for bank " & integer'image(b) severity error;
        elsif stat_done(b) = '0' then
          report "Memory test not completed for bank " & integer'image(b) severity error;
        end if;
      end loop;
      report "*** Simulation completed: TIMEOUT waiting for completion." severity failure;
    end if;
    wait;
  end process;

end architecture;