# Check that we are using 2015.4 or later
# See Xilinx quality alert at:
# http://www.xilinx.com/support/documentation/customer_notices/xcn15040.pdf
set viv_ver [version -short]
set viv_spl [split $viv_ver .]
set viv_maj [lindex $viv_spl 0]
set viv_min [lindex $viv_spl 1]

if { ($viv_maj == "2015" && $viv_min >= "4") || $viv_maj > "2015" } {
} else {
  puts "Vivado ${viv_ver} is not supported for model ADM-XRC-KU1."
  puts "Please review http://www.xilinx.com/support/documentation/customer_notices/xcn15040.pdf"
  return -1
}

# Get the directory containing this script. Source file paths are relative to this.
set origin_dir [file dirname [file normalize [info script]]]

# Define versions of dependencies in one convenient place
source "$origin_dir/version.tcl"

# Create project
set dir {dma_demo_pcie-ku060_2e}
set part {xcku060-ffva1517-2-e}
set ip_part {ku060_2e}
set name {dma_demo_pcie}
create_project -force -part $part $name "$origin_dir/$dir"

# Set the directory path for the new project
set proj_dir [get_property directory [current_project]]

# Set project properties
set project [get_projects $name]
set_property "default_lib" "xil_defaultlib" $project
set_property "part" $part $project
set_property "target_language" "VHDL" $project

set repo_paths [list \
  [file normalize "$origin_dir/../../../../fpga/repo/vivado-2014.4"] \
  [file normalize "$origin_dir/../../../../fpga/repo/vivado-2015.4"] \
]
set_property "ip_repo_paths" $repo_paths $project
update_ip_catalog


# Create 'sources_1' fileset (if not found)
if {[string equal [get_filesets -quiet sources_1] ""]} {
  create_fileset -srcset sources_1
}
set source_set [get_filesets sources_1]

# Add HDL source files
set hdl_files [list \
  [file normalize "$origin_dir/../src/adb3_admxrcku1_wrap.vhd"] \
  [file normalize "$origin_dir/../src/dma_demo_pcie.vhd"] \
]
if { [llength $hdl_files ] > 0 } {
  add_files -norecurse -fileset $source_set $hdl_files
}

# Create IP
create_ip -vlnv "alphadata:user:adb3_admxrcku1_x8_axi4:${bchi_ver}" -module_name adb3_admxrcku1_x8_axi4_ip
set adb3_admxrcku1_x8_axi4_ip [get_ips adb3_admxrcku1_x8_axi4_ip]
set_property -dict { \
  {CONFIG.use_diff_ref_clock} {true} \
  {CONFIG.number_of_dma_engines} {4} \
} $adb3_admxrcku1_x8_axi4_ip

create_ip -vlnv "alpha-data.com:user:ram_interface_axi4:${ram_if_axi_ver}" -module_name ram_interface_axi4_ip
set ram_interface_axi4_ip [get_ips ram_interface_axi4_ip]
set_property -dict { \
  CONFIG.ram_order {14} \
  CONFIG.ram_read_latency {3} \
  CONFIG.s_axi_axaddr_width {19} \
  CONFIG.s_axi_xdata_width {256} \
} $ram_interface_axi4_ip

create_ip -vlnv "xilinx.com:ip:axi_crossbar:*" -module_name axi_crossbar_ip
set axi_crossbar_ip [get_ips axi_crossbar_ip]
set_property -dict { \
  CONFIG.CLKIF.FREQ_HZ {250000000} \
  CONFIG.NUM_SI {1} \
  CONFIG.NUM_MI {4} \
  CONFIG.STRATEGY {2} \
  CONFIG.PROTOCOL {AXI4} \
  CONFIG.ADDR_WIDTH {64} \
  CONFIG.DATA_WIDTH {256} \
  CONFIG.ID_WIDTH {0} \
  CONFIG.CONNECTIVITY_MODE {SAMD} \
  CONFIG.ADDR_RANGES {1} \
  CONFIG.M00_A00_BASE_ADDR {0x0000000000000000} \
  CONFIG.M01_A00_BASE_ADDR {0x0000000000080000} \
  CONFIG.M02_A00_BASE_ADDR {0x0000000000100000} \
  CONFIG.M03_A00_BASE_ADDR {0x0000000000180000} \
  CONFIG.M00_A00_ADDR_WIDTH {19} \
  CONFIG.M01_A00_ADDR_WIDTH {19} \
  CONFIG.M02_A00_ADDR_WIDTH {19} \
  CONFIG.M03_A00_ADDR_WIDTH {19} \
} $axi_crossbar_ip

create_ip -vlnv "xilinx.com:ip:blk_mem_gen:*" -module_name blk_mem_gen_ip
set blk_mem_gen_ip [get_ips blk_mem_gen_ip]
set_property -dict { \
  CONFIG.AXI_Slave_Type {Memory_Slave}  \
  CONFIG.AXI_Type {AXI4_Full} \
  CONFIG.Additional_Inputs_for_Power_Estimation {false} \
  CONFIG.Algorithm {Minimum_Area} \
  CONFIG.Byte_Size {8} \
  CONFIG.CTRL_ECC_ALGO {NONE} \
  CONFIG.EN_SLEEP_PIN {false} \
  CONFIG.Enable_32bit_Address {false} \
  CONFIG.Enable_A {Use_ENA_Pin} \
  CONFIG.Enable_B {Use_ENB_Pin} \
  CONFIG.Fill_Remaining_Memory_Locations {false} \
  CONFIG.Interface_Type {Native} \
  CONFIG.Load_Init_File {false} \
  CONFIG.Memory_Type {True_Dual_Port_RAM} \
  CONFIG.Operating_Mode_A {WRITE_FIRST} \
  CONFIG.Operating_Mode_B {WRITE_FIRST} \
  CONFIG.Pipeline_Stages {0} \
  CONFIG.Port_A_Clock {100} \
  CONFIG.Port_A_Enable_Rate {100} \
  CONFIG.Port_A_Write_Rate {50} \
  CONFIG.Port_B_Clock {100} \
  CONFIG.Port_B_Enable_Rate {100} \
  CONFIG.Port_B_Write_Rate {50} \
  CONFIG.Read_Width_A {256} \
  CONFIG.Read_Width_B {256} \
  CONFIG.Register_PortA_Output_of_Memory_Core {true} \
  CONFIG.Register_PortA_Output_of_Memory_Primitives {true} \
  CONFIG.Register_PortB_Output_of_Memory_Core {true} \
  CONFIG.Register_PortB_Output_of_Memory_Primitives {true} \
  CONFIG.Use_Byte_Write_Enable {true} \
  CONFIG.Use_REGCEA_Pin {false} \
  CONFIG.Use_RSTA_Pin {false} \
  CONFIG.Use_RSTB_Pin {false} \
  CONFIG.Write_Depth_A {16384} \
  CONFIG.Write_Width_A {256} \
  CONFIG.Write_Width_B {256} \
  CONFIG.use_bram_block {Stand_Alone} \
} $blk_mem_gen_ip


# Create 'constrs_1' fileset (if not found)
if {[string equal [get_filesets -quiet constrs_1] ""]} {
  create_fileset -constrset constrs_1
}
set constraint_set [get_filesets constrs_1]

# Add constraints files
set constraint_files [list \
  [file normalize "$origin_dir/../src/refclk300.xdc"] \
  [file normalize "$origin_dir/../src/adb3_admxrcku1-pcie_x0y0.xdc"] \
  [file normalize "$origin_dir/../src/bitstream.xdc"] \
  [file normalize "$origin_dir/../src/dma_demo_pcie.xdc"] \
]
add_files -norecurse -fileset $constraint_set $constraint_files
set_property "target_constrs_file" [lindex $constraint_files end] $constraint_set


# Create 'sim_axi' fileset (if not found)
if {[string equal [get_filesets -quiet sim_axi] ""]} {
  create_fileset -simset sim_axi
}
set sim_set [get_filesets sim_axi]

# Add simulation HDL source files
set sim_files [list \
  [file normalize "$origin_dir/../src/axi4_256_pkg.vhd"] \
  [file normalize "$origin_dir/../src/axi4_256_tb_pkg.vhd"] \
  [file normalize "$origin_dir/../src/tb_dma_demo_pcie_stim.vhd"] \
  [file normalize "$origin_dir/../src/adb3_admxrcku1_sim.vhd"] \
  [file normalize "$origin_dir/../src/tb_dma_demo_pcie.vhd"] \
]
add_files -norecurse -fileset $sim_set $sim_files

# Set fileset properties
set_property "source_set" "sources_1" $sim_set
set_property "top" "tb_dma_demo_pcie" $sim_set
set_property "generic" "sim_axi=true" $sim_set

set_property "target_simulator" "ModelSim" $project
set_property "modelsim.simulate.runtime" "50 us" $sim_set
set_property "modelsim.simulate.custom_udo" [file normalize "$origin_dir/vsim_wave.udo"] $sim_set

set_property "target_simulator" "XSim" $project
set_property "xsim.simulate.runtime" "50 us" $sim_set
set_property "xsim.tclbatch" [file normalize "$origin_dir/xsim_wave.tcl"] $sim_set

set_property "simulator_language" "Mixed" $project

update_compile_order -fileset $sim_set


# Make sim_axi active simset
current_fileset -simset [get_filesets sim_axi]

if {![string equal [get_filesets -quiet sim_1] ""]} {
  delete_fileset sim_1
}


# Create 'synth_1' run (if not found)
if {[string equal [get_runs -quiet synth_1] ""]} {
  create_run -name synth_1 -part $part -flow {Vivado Synthesis 2015} -strategy "Vivado Synthesis Defaults" -constrset constrs_1
} else {
  set_property strategy "Vivado Synthesis Defaults" [get_runs synth_1]
  set_property flow "Vivado Synthesis 2015" [get_runs synth_1]
}
set synth_run [get_runs synth_1]
set_property "needs_refresh" "1" $synth_run
set_property "part" $part $synth_run
current_run -synthesis $synth_run


# Create 'impl_1' run (if not found)
if {[string equal [get_runs -quiet impl_1] ""]} {
  create_run -name impl_1 -part $part -flow {Vivado Implementation 2015} -strategy "Vivado Implementation Defaults" -constrset constrs_1 -parent_run synth_1
} else {
  set_property strategy "Vivado Implementation Defaults" [get_runs impl_1]
  set_property flow "Vivado Implementation 2015" [get_runs impl_1]
}
set impl_run [get_runs impl_1]
set_property "needs_refresh" "1" $impl_run
set_property "part" $part $impl_run

set_property -name {steps.route_design.args.more options} -value -timing_summary -objects $impl_run
set_property {steps.phys_opt_design.is_enabled} false $impl_run
set_property {steps.post_route_phys_opt_design.is_enabled} false $impl_run
current_run -implementation $impl_run

close_project

puts "INFO: Created project '$dir/$name'"
