--
-- This package defines parts for the MT40A family, currently:
--
-- MT40A2G4-083E
-- MT40A1G8-083E
-- MT40A512M16-083E
--
-- MT40A4G4-083E
-- MT40A2G8-083E
-- MT40A2G8-093E
--

library ieee;
use ieee.std_logic_1164.all;

library work;
use work.ddr4_sdram_model.all;

package ddr4_sdram_mt40a is

  constant PINOUT_MT40A_8G_X4 : pinout_t := (
    stack_type     => stack_none,
    stack_size     => 1,
    a_width        => 17,
    ba_width       => 2,
    bg_width       => 2,
    dq_nibbles     => 1,
    dqs_width      => 1,
    dm_dbi_width   => 1,
    dm_supported   => false,
    tdqs_supported => false,
    row_width      => 17,
    col_width      => 10
  );
    
  constant PINOUT_MT40A_8G_X4_TWIN_DIE : pinout_t := (
    stack_type     => stack_twin_die,
    stack_size     => 1,
    a_width        => 17,
    ba_width       => 2,
    bg_width       => 2,
    dq_nibbles     => 1,
    dqs_width      => 1,
    dm_dbi_width   => 1,
    dm_supported   => false,
    tdqs_supported => false,
    row_width      => 17,
    col_width      => 10
  );

  constant PINOUT_MT40A_8G_X8 : pinout_t := (
    stack_type     => stack_none,
    stack_size     => 1,
    a_width        => 17,
    ba_width       => 2,
    bg_width       => 2,
    dq_nibbles     => 2,
    dqs_width      => 1,
    dm_dbi_width   => 1,
    dm_supported   => true,
    tdqs_supported => true,
    row_width      => 16,
    col_width      => 10
  );

  constant PINOUT_MT40A_8G_X8_TWIN_DIE : pinout_t := (
    stack_type     => stack_twin_die,
    stack_size     => 1,
    a_width        => 17,
    ba_width       => 2,
    bg_width       => 2,
    dq_nibbles     => 2,
    dqs_width      => 1,
    dm_dbi_width   => 1,
    dm_supported   => true,
    tdqs_supported => true,
    row_width      => 16,
    col_width      => 10
  );
    
  constant PINOUT_MT40A_8G_X16 : pinout_t := (
    stack_type     => stack_none,
    stack_size     => 1,
    a_width        => 17,
    ba_width       => 2,
    bg_width       => 1,
    dq_nibbles     => 4,
    dqs_width      => 2,
    dm_dbi_width   => 2,
    dm_supported   => true,
    tdqs_supported => false,
    row_width      => 16,
    col_width      => 10
  );
  
  constant TIMING_MT40A_083E : timing_t := (
    power_on => (
      t_reset_hold    => 700 us,
      t_cke_reset_off => 10 ns
    ),
    reset => (
      t_reset_hold    => 100 ns,
      t_cke_reset_off => 10 ns
    ),
    init => (
      t_reset_cke     => 500 us,
      n_ck_cke        => 5,
      t_ck_cke        => 10 ns,
      n_xpr           => 5,
      t_xs_extra      => 10 ns,
      n_dllk          => 768,
      n_zqinit        => 1024
    ),
    operational => (
      n_mrd           => 8,
      t_mrd           => 0 ns,
      t_is            => 162 ps,
      t_ih            => 162 ps,
      t_rcd           => 13.32 ns,
      t_rp            => 13.32 ns
    ),
    refresh => (
      t_rfc           => 350 ns,
      t_refi          => 7.8 us,
      t_refi_ext      => 3.9 us
    )
  );

  constant TIMING_MT40A_093E : timing_t := (
    power_on => (
      t_reset_hold    => 700 us,
      t_cke_reset_off => 10 ns
    ),
    reset => (
      t_reset_hold    => 100 ns,
      t_cke_reset_off => 10 ns
    ),
    init => (
      t_reset_cke     => 500 us,
      n_ck_cke        => 5,
      t_ck_cke        => 10 ns,
      n_xpr           => 5,
      t_xs_extra      => 10 ns,
      n_dllk          => 768,
      n_zqinit        => 1024
    ),
    operational => (
      n_mrd           => 8,
      t_mrd           => 0 ns,
      t_is            => 180 ps,
      t_ih            => 180 ps,
      t_rcd           => 14.06 ns,
      t_rp            => 14.06 ns
    ),
    refresh => (
      t_rfc           => 350 ns,
      t_refi          => 7.8 us,
      t_refi_ext      => 3.9 us
    )
  );

  -- 8Gib density parts  
  constant MT40A2G4_083E : part_t := (
    timing => TIMING_MT40A_083E,
    pinout => PINOUT_MT40A_8G_X4
  );
  
  constant MT40A1G8_083E : part_t := (
    timing => TIMING_MT40A_083E,
    pinout => PINOUT_MT40A_8G_X8
  );
  
  constant MT40A512M16_083E : part_t := (
    timing => TIMING_MT40A_083E,
    pinout => PINOUT_MT40A_8G_X16
  );

  -- 16Gib density parts (twin die)
  constant MT40A4G4_083E : part_t := (
    timing => TIMING_MT40A_083E,
    pinout => PINOUT_MT40A_8G_X4_TWIN_DIE
  );
  
  constant MT40A2G8_083E : part_t := (
    timing => TIMING_MT40A_083E,
    pinout => PINOUT_MT40A_8G_X8_TWIN_DIE
  );
  
  constant MT40A2G8_093E : part_t := (
    timing => TIMING_MT40A_093E,
    pinout => PINOUT_MT40A_8G_X8_TWIN_DIE
  );  
  
end package;

package body ddr4_sdram_mt40a is


end package body;
