#ifndef _ADATA_PLATFORM_EVENT_H
#define _ADATA_PLATFORM_EVENT_H

#include <stdlib.h>
#include <assert.h>
#include <errno.h>
#include <semaphore.h>

#define PORTABLE_EVENT_HANDLE_INVALID_VALUE NULL
typedef sem_t* PortableEventHandle;

typedef enum _PortableEventStatus {
  PortableEventOk = 0,
  PortableEventTimeout = 1,
  PortableEventFailed = 2
} PortableEventStatus;

inline static PortableEventHandle
portableEventCreate(
  void)
{
  sem_t* pSem;
  int err;
  
  pSem = malloc(sizeof(*pSem));
  if (NULL == pSem) {
    return NULL;
  }
  err = sem_init(pSem, 0, 0);
  assert(err == 0);
  return pSem;
}

inline static void
portableEventDestroy(
  PortableEventHandle e)
{
  sem_destroy(e);
  free(e);
}

inline static void
portableEventSignal(
  PortableEventHandle e)
{
  sem_post(e);
}

inline static PortableEventStatus
portableEventWait(
  PortableEventHandle e)
{
  if (0 == sem_wait(e)) {
    return PortableEventOk;
  } else {
    return PortableEventFailed;
  }
}

inline static PortableEventStatus
portableEventWaitTimeout(
  PortableEventHandle e,
  unsigned int timeoutMs)
{
  struct timespec to;
  int err;
  
  to.tv_sec = timeoutMs / 1000;
  to.tv_nsec = 1000000 * (timeoutMs % 1000);
  err = sem_timedwait(e, &to);
  switch (err) {
  case 0:
    return PortableEventOk;
  
  case ETIMEDOUT:
    return PortableEventTimeout;
    
  default:
    return PortableEventFailed;
  }
}

#endif
