#ifndef _ADATA_COMMONTHREAD_H
#define _ADATA_COMMONTHREAD_H

#include <errno.h>
#include <pthread.h>
#include <stdlib.h>
#include <time.h>

typedef void* (PortableThreadFunction)(void*);

typedef struct _PortableThread {
  PortableThreadFunction* pFunction;
  pthread_t thread;
  void* pArgument;
} PortableThread;

static void*
_portableThreadStub(
  void* pArgument)
{
  PortableThread* pThread = (PortableThread*)pArgument;
  int oldType;
  void* pValue;

  if (0 != pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS, &oldType)) {
    fprintf(stderr, "_portableThreadStub: failed to set cancel type to ASYNCHRONOUS\n");
  }
  pValue = pThread->pFunction(pThread->pArgument);
  pthread_exit(pValue);
  return pValue;
}

__inline static PortableThread*
portableThreadCreate(
  PortableThreadFunction* pFunction,
  void* pArgument)
{
  PortableThread* pNew;
  int res;

  pNew = (PortableThread*)malloc(sizeof(*pNew));
  if (NULL == pNew) {
    return NULL;
  }
  pNew->pFunction = pFunction;
  pNew->pArgument = pArgument;

  res = pthread_create(&pNew->thread, NULL, _portableThreadStub, (void*)pNew);
  if (0 != res) {
    free(pNew);
    return NULL;
  }
  return pNew;
}

__inline static int
portableThreadWaitInfinite(
  PortableThread* pThread,
  void** ppResult)
{
  void* pResult;

  if (0 == pthread_join(pThread->thread, &pResult)) {
    if (NULL != ppResult) {
      *ppResult = pResult;
    }
    return TRUE;
  }
  return FALSE;
}

__inline static int
portableThreadWait(
  PortableThread* pThread,
  unsigned int timeoutMs,
  void** ppResult)
{
  void* pResult;
  struct timespec timeout;

  clock_gettime(CLOCK_REALTIME, &timeout);
  timeout.tv_sec += timeoutMs / 1000;
  timeout.tv_nsec += (timeoutMs % 1000) * 1000000;
  if (timeout.tv_nsec >= 1000000000) {
    timeout.tv_nsec -= 1000000000;
    timeout.tv_sec++;
  }
  if (0 == pthread_timedjoin_np(pThread->thread, &pResult, &timeout)) {
    if (NULL != ppResult) {
      *ppResult = pResult;
    }
    return TRUE;
  }
  return FALSE;
}

__inline static int
portableThreadKill(
  PortableThread* pThread)
{
  void* pResult;
  struct timespec timeout;
  int res;

  if (0 == pthread_cancel(pThread->thread)) {
    clock_gettime(CLOCK_REALTIME, &timeout);
    /* Wait 2 seconds for thread to be killed */
    timeout.tv_sec += 2;
    res = pthread_timedjoin_np(pThread->thread, &pResult, &timeout);
    if (0 == res) {
      return TRUE;
    }
  }
  return FALSE;
}

#endif
