using namespace std;

#include <dlfcn.h>

#include <iostream>
#include <iomanip>
#include <sstream>

#include "../../sdk_version.h"
#include "devicestatuswnd.h"

#if (GTK_MAJOR_VERSION >= 3)
# define APPEND_TEXT(ctl, text) ((ctl).append(text))
#else
# define APPEND_TEXT(ctl, text) ((ctl).append_text(text))
#endif

// -------------------------------------------------------------------------
// Start of IDeviceStatusWnd interface
// -------------------------------------------------------------------------

void
CDeviceStatusWnd::deviceChanged()
{
  const DeviceInfo& devInfo = m_pParent->getDeviceInfo();
  std::ostringstream ss;

  if (!m_bFunctionalityAvailable) {
    return;
  }

  for (unsigned int i = 0; i < 5; i++) {
    m_rawListCtl.set_text(i, 1, "");
  }
  m_decListCtl.clear_items();
  if (devInfo.m_hDevice != ADMXRC3_HANDLE_INVALID_VALUE) {
    for (unsigned int i = 0; i < devInfo.m_deviceInfo.NumWindow; i++) {
      ss.str("");
      ss << "Window " << i << " timeout";
      APPEND_TEXT(m_decListCtl, ss.str());
    }
    for (unsigned int i = 0; i < devInfo.m_deviceInfo.NumDmaChannel; i++) {
      ss.str("");
      ss << "DMA engine " << i << " bus timeout";
      APPEND_TEXT(m_decListCtl, ss.str());
    }
    APPEND_TEXT(m_decListCtl, "Direct Master bus timeout");
    APPEND_TEXT(m_decListCtl, "Direct Master bad transaction");
  }

  m_clearCtl.set_sensitive((devInfo.m_hDevice != ADMXRC3_HANDLE_INVALID_VALUE /* && m_bInAdminGroup */) ? TRUE : FALSE);

  update();
}

bool_t
CDeviceStatusWnd::doClearDeviceErrors()
{
  const DeviceInfo& devInfo = m_pParent->getDeviceInfo();
  ADMXRC3_HANDLE hDevice = ADMXRC3_HANDLE_INVALID_VALUE;
  ADMXRC3_STATUS status;

  if (!m_bFunctionalityAvailable) {
    std::ostringstream ss;
    ss.str("");
    ss << "Fuctionality not available in driver";
    Gtk::MessageDialog errorDialog(m_pParent->getNativeWindow(), ss.str(), false, Gtk::MESSAGE_ERROR, Gtk::BUTTONS_CLOSE, true);
    errorDialog.set_title("Failed to clear device errors");
    errorDialog.run();
    return FALSE;
  }

  if (devInfo.m_hDevice != ADMXRC3_HANDLE_INVALID_VALUE) {
    status = ADMXRC3_OpenEx(devInfo.m_deviceIndex, FALSE, 0, &hDevice);
    if (ADMXRC3_SUCCESS != status) {
      std::ostringstream ss;
      ss.str("");
      ss << "Failed to reopen device in active mode\n\n";
      ss << "Index=" << devInfo.m_deviceIndex << " Status=0x" << HEX << status << DEC;
      ss << "(" << ADMXRC3_GetStatusString(status, TRUE) << ")";
      Gtk::MessageDialog errorDialog(m_pParent->getNativeWindow(), ss.str(), false, Gtk::MESSAGE_ERROR, Gtk::BUTTONS_CLOSE, true);
      errorDialog.set_title("Failed to clear device errors");
      errorDialog.run();
      return FALSE;
    }

    status = m_pAdmxrc3ClearDeviceErrors(hDevice);
    if (ADMXRC3_SUCCESS != status) {
      std::ostringstream ss;
      ss.str("");
      ss << "Failed to call ADMXRC3_ClearDeviceErrors\n\n";
      ss << "Index=" << devInfo.m_deviceIndex << " Status=0x" << HEX << status << DEC;
      ss << "(" << ADMXRC3_GetStatusString(status, TRUE) << ")";
      Gtk::MessageDialog errorDialog(m_pParent->getNativeWindow(), ss.str(), false, Gtk::MESSAGE_ERROR, Gtk::BUTTONS_CLOSE, true);
      errorDialog.set_title("Failed to clear device errors");
      errorDialog.run();
      ADMXRC3_Close(hDevice);
      return FALSE;
    }

    ADMXRC3_Close(hDevice);

    update();

    return TRUE;
  } else {
    std::ostringstream ss;
    ss.str("");
    ss << "No device selected";
    Gtk::MessageDialog errorDialog(m_pParent->getNativeWindow(), ss.str(), false, Gtk::MESSAGE_ERROR, Gtk::BUTTONS_CLOSE, true);
    errorDialog.set_title("Failed to clear device errors");
    errorDialog.run();
    return FALSE;
  }
}

void
CDeviceStatusWnd::numDeviceChanged()
{
  unsigned int numDevice = m_pParent->getDeviceInfo().m_numDevice;

  if ((m_numDevice == 0) == (numDevice == 0)) {
    return;
  }

  /* Nothing to do yet */

  m_numDevice = numDevice;
}

void
CDeviceStatusWnd::update()
{
  const DeviceInfo& devInfo = m_pParent->getDeviceInfo();
  ADMXRC3_DEVICE_STATUS devStatus;
  ADMXRC3_STATUS status;
  unsigned int j;
  std::ostringstream ss;

  if (!m_bFunctionalityAvailable) {
    return;
  }

  if (devInfo.m_hDevice != ADMXRC3_HANDLE_INVALID_VALUE) {
    status = m_pAdmxrc3GetDeviceStatus(devInfo.m_hDevice, 0, &devStatus, NULL);
    if (ADMXRC3_SUCCESS != status) {
      for (unsigned int i = 0; i < 5; i++, j++) {
        m_rawListCtl.set_text(i, 1, "?");
      }
      j = 0;
      for (unsigned int i = 0; i < devInfo.m_deviceInfo.NumWindow; i++, j++) {
        m_decListCtl.set_text(j, 1, "?");
      }
      for (unsigned int i = 0; i < devInfo.m_deviceInfo.NumDmaChannel; i++, j++) {
        m_decListCtl.set_text(j, 1, "?");
      }
      for (unsigned int i = 0; i < 2; i++, j++) {
        m_decListCtl.set_text(j, 1, "?");
      }
    } else {
      j = 0;
      ss.str("");
      ss << "0x" << HEX << std::setfill('0') << std::setw(8) << devStatus.Window << DEC;
      m_rawListCtl.set_text(j++, 1, ss.str());
      ss.str("");
      ss << "0x" << HEX << std::setfill('0') << std::setw(8) << devStatus.DMA << DEC;
      m_rawListCtl.set_text(j++, 1, ss.str());
      ss.str("");
      ss << "0x" << HEX << std::setfill('0') << std::setw(8) << devStatus.DirectMaster << DEC;
      m_rawListCtl.set_text(j++, 1, ss.str());
      ss.str("");
      ss << "0x" << HEX << std::setfill('0') << std::setw(8) << devStatus.ModelLow << DEC;
      m_rawListCtl.set_text(j++, 1, ss.str());
      ss.str("");
      ss << "0x" << HEX << std::setfill('0') << std::setw(8) << devStatus.ModelHigh << DEC;
      m_rawListCtl.set_text(j++, 1, ss.str());
      j = 0;
      for (unsigned int i = 0; i < devInfo.m_deviceInfo.NumWindow; i++, j++) {
        ss.str("");
        ss << ((ADMXRC3_DS_LOCAL_TIMEOUT(i) & devStatus.Window) ? 1U : 0U);
        m_decListCtl.set_text(j, 1, ss.str());
      }
      for (unsigned int i = 0; i < devInfo.m_deviceInfo.NumDmaChannel; i++, j++) {
        ss.str("");
        ss << ((ADMXRC3_DMA_BUS_TIMEOUT(i) & devStatus.DMA) ? 1U : 0U);
        m_decListCtl.set_text(j, 1, ss.str());
      }
      ss.str("");
      ss << ((ADMXRC3_DM_BUS_TIMEOUT & devStatus.DirectMaster) ? 1U : 0U);
      m_decListCtl.set_text(j++, 1, ss.str());
      ss.str("");
      ss << ((ADMXRC3_DM_BAD_TRANSACTION & devStatus.DirectMaster) ? 1U : 0U);
      m_decListCtl.set_text(j++, 1, ss.str());
    }
  }
}

// -------------------------------------------------------------------------
// End of IDeviceStatusWnd interface
// -------------------------------------------------------------------------

CDeviceStatusWnd::CDeviceStatusWnd(
  ISysMonWnd* pParent,
  bool_t bVisible) : m_rawListCtl(2), m_decListCtl(2)
{
  const DeviceInfo& devInfo = pParent->getDeviceInfo();

  m_pParent = pParent;
  m_numDevice = 0;
  m_bFunctionalityAvailable = FALSE; /* Assume FALSE, may set to TRUE later */

  /*
  ** We dynamically get the entry points for ADMXRC3_GetDeviceStatus and
  ** ADMXRC3_ClearDeviceErrors so that if the ADMXRC3 runtime installed on
  ** the system happens to predate the addition of these functions to the
  ** ADMXRC3 API, we can gracefully disable the corresponding functionality
  ** in this application.
  */
  m_pAdmxrc3So = dlopen("libadmxrc3.so", RTLD_LAZY | RTLD_NOLOAD);
  if (NULL != m_pAdmxrc3So) {
    m_pAdmxrc3GetDeviceStatus = (ADMXRC3_GETDEVICESTATUS*)dlsym(m_pAdmxrc3So, "ADMXRC3_GetDeviceStatus");
    m_pAdmxrc3ClearDeviceErrors = (ADMXRC3_CLEARDEVICEERRORS*)dlsym(m_pAdmxrc3So, "ADMXRC3_ClearDeviceErrors");
    if (NULL != m_pAdmxrc3GetDeviceStatus && NULL != m_pAdmxrc3ClearDeviceErrors) {
      m_bFunctionalityAvailable = TRUE;
    } else {
      dlclose(m_pAdmxrc3So);
      m_pAdmxrc3So = NULL;
    }
  }

  set_size_request(-1, 300);

  if (m_bFunctionalityAvailable) {
    m_rawLabelCtl.set_label("Device status");
    m_rawListCtl.set_column_title(0, "Field");
    m_rawListCtl.set_column_title(1, "Value");
    APPEND_TEXT(m_rawListCtl, "Window");
    APPEND_TEXT(m_rawListCtl, "DMA");
    APPEND_TEXT(m_rawListCtl, "DirectMaster");
    APPEND_TEXT(m_rawListCtl, "ModelLow");
    APPEND_TEXT(m_rawListCtl, "ModelHigh");

    m_rawScrollCtl.add(m_rawListCtl);
    m_rawScrollCtl.set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);

    m_rawFrameCtl.add(m_rawScrollCtl);

    m_rawVboxCtl.set_spacing(8);
    m_rawVboxCtl.pack_start(m_rawLabelCtl, FALSE, FALSE);
    m_rawVboxCtl.pack_start(m_rawFrameCtl, TRUE, TRUE);

    m_decLabelCtl.set_label("Decoded device status");
    m_decListCtl.set_column_title(0, "Error type");
    m_decListCtl.set_column_title(1, "Value");

    m_decScrollCtl.add(m_decListCtl);
    m_decScrollCtl.set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);

    m_decFrameCtl.add(m_decScrollCtl);

    m_decVboxCtl.set_spacing(8);
    m_decVboxCtl.pack_start(m_decLabelCtl, FALSE, FALSE);
    m_decVboxCtl.pack_start(m_decFrameCtl, TRUE, TRUE);

    m_hboxCtl.set_homogeneous(TRUE);
    m_hboxCtl.set_spacing(40);
    m_hboxCtl.add(m_rawVboxCtl);
    m_hboxCtl.add(m_decVboxCtl);

    m_clearCtl.set_label("Clear Errors");
    m_clearCtl.signal_clicked().connect(sigc::mem_fun(*this, &CDeviceStatusWnd::onButtonClear));
    m_clearCtl.set_sensitive((devInfo.m_hDevice != ADMXRC3_HANDLE_INVALID_VALUE /* && m_bInAdminGroup */) ? TRUE : FALSE);

    m_clearLabelCtl.set_label("If executing as root, device errors can be cleared");

    m_clearHboxCtl.set_spacing(8);
    m_clearHboxCtl.pack_start(m_clearCtl, FALSE, FALSE);
    m_clearHboxCtl.pack_start(m_clearLabelCtl, FALSE, FALSE);

    m_vboxCtl.set_border_width(10);
    m_vboxCtl.set_spacing(8);
    m_vboxCtl.pack_start(m_hboxCtl, TRUE, TRUE);
    m_vboxCtl.pack_end(m_clearHboxCtl, FALSE, FALSE);

    add(m_vboxCtl);
  } else {
    m_notAvailableCtl.set_label("Functionality not available in driver");
    add(m_notAvailableCtl);
  }
}

CDeviceStatusWnd::~CDeviceStatusWnd()
{
  if (NULL != m_pAdmxrc3So) {
    dlclose(m_pAdmxrc3So);
  }
}

void
CDeviceStatusWnd::onButtonClear()
{
  doClearDeviceErrors();
}
