#pragma once

#include <objmodel.h>
#include "stripedline.h"

class CTraceCheckButton : public Gtk::HBox
{
public:
  CTraceCheckButton() {
    m_pLineRenderer = NULL;
    m_drawing.set_size_request(50, 0);
#if (GTK_MAJOR_VERSION >= 3)
    m_drawing.signal_draw().connect(sigc::mem_fun(*this, &CTraceCheckButton::onDraw), false);
#else
    m_drawing.signal_expose_event().connect(sigc::mem_fun(*this, &CTraceCheckButton::onExpose), false);
#endif
    set_spacing(6);
    pack_start(m_drawing, FALSE, FALSE);
    pack_start(m_button, FALSE, FALSE);
  }

  Gtk::CheckButton& getButton() {
    return m_button;
  }

  CStripedLine* getRenderer() {
    return m_pLineRenderer;
  }

  void setColor(const color_t m_colors[2]) {
    m_traceColor[0] = m_colors[0];
    m_traceColor[1] = m_colors[1];
  }

  void setRenderer(CStripedLine* pRenderer) {
    m_pLineRenderer = pRenderer;
  }

protected:
  CStripedLine* m_pLineRenderer;
  Gtk::DrawingArea m_drawing;
  Gtk::CheckButton m_button;
  color_t m_traceColor[2];

protected:
  bool onDraw(const Cairo::RefPtr<Cairo::Context>& cr) {
    static const float timeScale = 1.0f / 12.0f;
    Gtk::Allocation allocation = m_drawing.get_allocation();
    double w = (double)allocation.get_width();
    double h = (double)allocation.get_height();

    cr->set_source_rgb(0.0, 0.0, 0.0);
    cr->paint();

    cr->set_antialias(Cairo::ANTIALIAS_NONE);
    cr->set_line_width(1.0);

    if (NULL != m_pLineRenderer) {
      m_pLineRenderer->setContext(cr);
      m_pLineRenderer->setColors(2, m_traceColor);
      m_pLineRenderer->start(10.0f, h * 0.5f, 0.0f);
      m_pLineRenderer->add(w - 10.0f, h * 0.5f, (w - 20.0f) * timeScale);
      m_pLineRenderer->finish();
    }

    return false;
  }

  bool onExpose(GdkEventExpose* pEvent) {
    Glib::RefPtr<Gdk::Window> window = m_drawing.get_window();
    if (window) {
      Cairo::RefPtr<Cairo::Context> cr = window->create_cairo_context();
      return onDraw(cr);
    }
    return false;
  }
};

class CScopeKey : public Gtk::Window, public IScopeKey
{
public:
	CScopeKey(IGraphWndToolbar* pParent);
	virtual ~CScopeKey();
  // IScopeKey interface
  window_handle_t getNativeWindow() { return *this; }
  void tracesChanged();

protected:
  IGraphWndToolbar* m_pParent;
  CStripedLine m_lineRenderer;
  Gtk::Button m_allCtl;
  Gtk::Button m_noneCtl;
  Gtk::HButtonBox m_allNoneCtl;
  Gtk::VBox m_tableCtl;
  Gtk::VBox m_vboxCtl;
  CTraceCheckButton* m_pCheckButtons;
  unsigned int m_numTrace;
  bool m_bFirstShow;
  gint m_x, m_y;

protected:
  void deleteButtons();
  void onButtonAll();
  void onButtonNone();
  void onToggle(unsigned int index);
  void on_show();
  void on_hide();
};
