#include "stdafx.h"
#include "afxwinappex.h"

#ifdef _DEBUG
# define new DEBUG_NEW
#endif

#include <platform.h>
#include <objmodel.h>

#include "control_id.h"
#include "clockgenwnd.h"
#include "uachelp.h"

IMPLEMENT_DYNAMIC(CEnterEdit, CEdit)

BEGIN_MESSAGE_MAP(CEnterEdit, CEdit)
  ON_WM_KEYDOWN()
END_MESSAGE_MAP()

bool
CEnterEdit::ValidateText()
{
  CString text;
  double dummy;
  GetWindowText(text);
  if (_stscanf_s(text.GetBuffer(), _T("%lf"), &dummy) == 1) {
    return true;
  }
  return false;
}

void
CEnterEdit::OnKeyDown(
  UINT nChar,
  UINT nRepCnt,
  UINT nFlags)
{
  if (nChar == VK_RETURN || nChar == VK_TAB) {
    ShowWindow(SW_HIDE);
    if (ValidateText()) {
      GetParent()->SendMessage(WM_SYSMON_EDITFREQ, 0, 0);
    } else {
      MessageBox(_T("Frequency must be a decimal or floating point number"), _T("Not a valid frequency"), MB_ICONEXCLAMATION | MB_OK);
    }
  } else if (nChar == VK_ESCAPE) {
    ShowWindow(SW_HIDE);
  } else {
    CEdit::OnKeyDown(nChar, nRepCnt, nFlags);
  }
}

IMPLEMENT_DYNAMIC(CClockGenWnd, CWnd)

BEGIN_MESSAGE_MAP(CClockGenWnd, CWnd)
  ON_WM_CREATE()
  ON_WM_SIZE()
  ON_WM_CTLCOLOR()
  ON_COMMAND(IDCTL_ALLOWFREQCHANGES, OnButtonAllowChanges)
  ON_NOTIFY(NM_DBLCLK, IDCTL_CLOCKGEN_LIST, OnListDblClk)
  ON_EN_SETFOCUS(IDCTL_FREQ_EDIT, OnFreqEditSetFocus)
  ON_EN_KILLFOCUS(IDCTL_FREQ_EDIT, OnFreqEditKillFocus)
  ON_MESSAGE(WM_SYSMON_EDITFREQ, OnFreqEditOk)
END_MESSAGE_MAP()

static const float g_fMinTotalWidth = 580;
static const float g_fColumnIndex = 0.07f;
static const float g_fColumnFreq = 0.30f;
static const float g_fColumnDescription = 1.0f - g_fColumnIndex - g_fColumnFreq;

// -------------------------------------------------------------------------
// Start of ISensorInfoWnd interface
// -------------------------------------------------------------------------

void
CClockGenWnd::deviceChanged()
{
  m_listCtl.SetRedraw(FALSE);
  const DeviceInfo& devInfo = m_pParent->getDeviceInfo();
  if (devInfo.m_hDevice != ADMXRC3_HANDLE_INVALID_VALUE) {
    m_listCtl.DeleteAllItems();
  }
  if (ADMXRC3_HANDLE_INVALID_VALUE != devInfo.m_hDevice) {
    m_listCtl.InsertItem(0, _T("#"));
    m_listCtl.SetItem(0, 1, LVIF_TEXT, _T("Description"), 0, 0, 0, (LPARAM)0);
    m_listCtl.SetItem(0, 2, LVIF_TEXT, _T("Frequency (MHz)"), 0, 0, 0, (LPARAM)0);
    m_listCtl.InsertItem(1, _T(""));
    for (unsigned int i = 0; i < devInfo.m_numClockGen; i++) {
      CString str;
      unsigned int j = i + 2;
      str.Format(_T("%u"), i);
      m_listCtl.InsertItem(j, str);
      const TCHAR* pTmp;
      pTmp = CAdmxrc3Help::getClockGenDescription(i, devInfo.m_deviceInfo.Model);
      if (NULL == pTmp) {
        m_listCtl.SetItem(j, 1, LVIF_TEXT, _T("Unknown"), 0, 0, 0, (LPARAM)0);
      } else {
        m_listCtl.SetItem(j, 1, LVIF_TEXT, pTmp, 0, 0, 0, (LPARAM)0);
      }
    }
  }
  m_listCtl.SetRedraw(TRUE);

  m_allowChangeCtl.EnableWindow((devInfo.m_hDevice != ADMXRC3_HANDLE_INVALID_VALUE && m_bInAdminGroup) ? TRUE : FALSE);
}

void
CClockGenWnd::numDeviceChanged()
{
  unsigned int numDevice = m_pParent->getDeviceInfo().m_numDevice;

  if ((m_numDevice == 0) == (numDevice == 0)) {
    return;
  }

  /* Nothing to do yet */

  m_numDevice = numDevice;
}

void
CClockGenWnd::update()
{
  const DeviceInfo& devInfo = m_pParent->getDeviceInfo();
  unsigned int n = devInfo.m_numClockGen;
  ADMXRC3_STATUS status;
  double frequency;

  m_listCtl.SetRedraw(FALSE);
  for (unsigned int i = 0; i < n; i++) {
    status = ADMXRC3_GetClockFrequency(devInfo.m_hDevice, i, &frequency);
    if (ADMXRC3_SUCCESS == status) {
      CString str;
      str.Format(_T("%.3f"), (float)(frequency / 1.0e6));
      m_listCtl.SetItem(i + 2, 2, LVIF_TEXT, str, 0, 0, 0, (LPARAM)0);
    } else {
      const TCHAR* pErrorStr = ADMXRC3_GetStatusString(status, TRUE);
      m_listCtl.SetItem(i + 2, 2, LVIF_TEXT, pErrorStr, 0, 0, 0, (LPARAM)0);
    }
  }
  m_listCtl.SetRedraw(TRUE);
}

// -------------------------------------------------------------------------
// End of ISensorInfoWnd interface
// -------------------------------------------------------------------------

CClockGenWnd::CClockGenWnd(
  ISysMonWnd* pParent,
  bool_t bVisible)
{
  const TCHAR* pWndClass = NULL;
  bool_t bSuccess;

  m_pParent = pParent;
  m_numDevice = 0;
  m_bIsElevated = IsRunAsAdmin();

  m_bgBrush.CreateSysColorBrush(COLOR_BTNFACE);
  m_font.CreateStockObject(DEFAULT_GUI_FONT);

  pWndClass = AfxRegisterWndClass(0, ::LoadCursor(NULL, IDC_ARROW), NULL, NULL);
  bSuccess = CreateEx(WS_EX_CONTROLPARENT, pWndClass, _T(""), WS_CHILD | WS_CLIPCHILDREN | (bVisible ? WS_VISIBLE : 0), 0, 0, 1, 1, m_pParent->getNativeWindow(), NULL, NULL);
  ASSERT(bSuccess);
}

CClockGenWnd::~CClockGenWnd()
{
}

int
CClockGenWnd::OnCreate(
  LPCREATESTRUCT lpCreateStruct)
{
  const DeviceInfo& devInfo = m_pParent->getDeviceInfo();
  RECT r = { 0, 0, 0, 0 };

  if (CWnd::OnCreate(lpCreateStruct) == -1) {
    return -1;
  }

  m_listCtl.CreateEx(WS_EX_STATICEDGE, WS_CHILD | WS_CLIPSIBLINGS | WS_VISIBLE | WS_TABSTOP | LVS_REPORT | LVS_NOCOLUMNHEADER | LVS_SINGLESEL, r, this, IDCTL_CLOCKGEN_LIST);
  m_listCtl.SetExtendedStyle(LVS_EX_GRIDLINES | LVS_EX_INFOTIP | LVS_EX_FULLROWSELECT);
  m_listCtl.SetFont(&m_font);
  m_listCtl.InsertColumn(0, _T("#"), LVCFMT_LEFT, (int)(g_fMinTotalWidth * g_fColumnIndex), -1);
  m_listCtl.InsertColumn(1, _T("Description"), LVCFMT_LEFT, (int)(g_fMinTotalWidth * g_fColumnDescription), -1);
  m_listCtl.InsertColumn(2, _T("Frequency"), LVCFMT_LEFT, (int)(g_fMinTotalWidth * g_fColumnFreq), -1);

  /* Create the edit control for entering a new frequency */
  m_freqEditCtl.Create(WS_CHILD | WS_CLIPSIBLINGS | ES_LEFT | ES_MULTILINE, r, this, IDCTL_FREQ_EDIT);
  m_freqEditCtl.SetFont(&m_font);

  /* Create the "Allow Changes" button */
  m_allowChangeCtl.Create(
    _T("Allow Changes"),
    WS_VISIBLE | WS_CHILD | WS_CLIPSIBLINGS | WS_TABSTOP,
    r,
    this,
    IDCTL_ALLOWFREQCHANGES);
  m_allowChangeCtl.SetFont(&m_font);
  m_allowChangeCtl.EnableWindow(FALSE);

  /* Attempt to add UAC shield; will fail if < Vista, but we don't care. */
#define BCM_SETSHIELD (BCM_FIRST + 0x000C)
  ::SendMessage(m_allowChangeCtl.m_hWnd, (UINT)BCM_SETSHIELD, (WPARAM)0, (LPARAM)TRUE);

  m_allowChangeLabelCtl.Create(_T(""), WS_VISIBLE | WS_CHILD | SS_LEFT | WS_VISIBLE, r, this, (UINT)IDC_STATIC);
  m_allowChangeLabelCtl.SetFont(&m_font);

  if (m_bIsElevated) {
    m_allowChangeLabelCtl.SetWindowText(_T("Double click on a frequency to change it."));
  } else {
    /* Figure out if we are in Administrators group, and enable the "Clear Errors" button if yes. */
    try {
      m_bInAdminGroup = IsUserInAdminGroup();
      if (!m_bInAdminGroup) {
        m_allowChangeLabelCtl.SetWindowText(_T("Function unavailable because not in Administrators group."));
      } else {
        if (devInfo.m_hDevice != ADMXRC3_HANDLE_INVALID_VALUE) {
          m_allowChangeCtl.EnableWindow(TRUE);
        }
      }
    } catch (DWORD dwError) {
      CString str;
      str.Format(_T("Function unavailable because unable to determine if in Administrators group. Error=0x%x\n"), dwError);
      m_allowChangeLabelCtl.SetWindowText(str);
    }
  }

  return 0;
}

void
CClockGenWnd::OnSize(
  UINT nType,
  int cx,
  int cy)
{
  CWnd::OnSize(nType, cx, cy);

  m_listCtl.MoveWindow(0, 0, cx, cy - 32);
  m_allowChangeCtl.MoveWindow(0, cy - 23, 110, 23);
  m_allowChangeLabelCtl.MoveWindow(120, cy - 18, cx - 120, 18);
}

HBRUSH
CClockGenWnd::OnCtlColor(
  CDC* pDC,
  CWnd* pWnd,
  UINT nCtlColor)
{
  if (pWnd == &m_allowChangeLabelCtl) {
    pDC->SetBkMode(TRANSPARENT);
    return (HBRUSH)GetStockObject(HOLLOW_BRUSH);
  }

  return CWnd::OnCtlColor(pDC, pWnd, nCtlColor);
}

void
CClockGenWnd::OnButtonAllowChanges()
{
  const DeviceInfo& devInfo = m_pParent->getDeviceInfo();
  CString msg, param;
  TCHAR szPath[MAX_PATH];
  SHELLEXECUTEINFO sei;

  ASSERT(!m_bIsElevated);

  /* Re-launch ourself elevated. */
  if (GetModuleFileName(NULL, szPath, ARRAYSIZE(szPath))) {
    ZeroMemory(&sei, sizeof(sei));
    sei.cbSize = sizeof(sei);
    sei.lpVerb = _T("runas");
    sei.lpFile = szPath;
    /*
    ** This command line causes (in this order):
    ** 1. The current device to be selected.
    ** 2. The "Clock Generators" tab to be selected.
    */
    param.Format(_T("-verb sel_dev_idx %u -verb sel_tab 4"), devInfo.m_deviceIndex);
    sei.lpParameters = param;
    sei.hwnd = m_hWnd;
    sei.nShow = SW_NORMAL;
    if (!ShellExecuteEx(&sei)) {
      DWORD dwError = GetLastError();

      if (dwError == ERROR_CANCELLED) {
        /* The user refused the elevation; do nothing more. */
      } else {
        msg.Format(_T("File='%s'\nError=0x%lx"), szPath, (unsigned long)dwError);
        MessageBox(msg, _T("Failed to re-launch as Administrator"), MB_ICONEXCLAMATION | MB_OK);
      }
    } else {
      m_pParent->requestQuit();
    } 
  } else {
    msg.Format(_T("Win32 error 0x%lx\n"), (unsigned long)GetLastError());
    MessageBox(msg, _T("Failed to get own executable filename"), MB_ICONEXCLAMATION | MB_OK);
  }
}

void
CClockGenWnd::OnListDblClk(
  NMHDR* pNotifyStruct,
  LRESULT* /* pResult */)
{
  NMITEMACTIVATE* pActivate = (NMITEMACTIVATE*)pNotifyStruct;

  if (!m_bIsElevated || /* Can't change frequency because not running elevated */
    pActivate->iItem < 2 || /* First two rows show column names and an empty line, respectively */
    pActivate->iSubItem != 2) /* 3rd column is Frequency */
  {
    return;
  }
  unsigned int index = (unsigned int)pActivate->iItem - 2;
  const DeviceInfo& devInfo = m_pParent->getDeviceInfo();
  if (index >= devInfo.m_numClockGen) {
    return;
  }

  CString text;
  RECT r, listRect, wndRect;
  ADMXRC3_STATUS status;
  double frequency;

  ListView_GetSubItemRect(m_listCtl.m_hWnd, pActivate->iItem, pActivate->iSubItem, LVIR_LABEL, &r);
  status = ADMXRC3_GetClockFrequency(devInfo.m_hDevice, index, &frequency);
  if (ADMXRC3_SUCCESS == status) {
    text.Format(_T("%f"), (float)(frequency / 1.0e6));
  }
  m_listCtl.GetWindowRect(&listRect);
  this->GetWindowRect(&wndRect);
  m_freqEditCtl.SetWindowText(text);
  m_freqEditCtl.ShowWindow(SW_SHOW);
  ::SetWindowPos(m_freqEditCtl.m_hWnd, HWND_TOP, 8 + r.left + listRect.left - wndRect.left, 1 + r.top + listRect.top - wndRect.top, r.right - r.left - 13, r.bottom - r.top - 1, 0);
  m_freqEditCtl.SetSel(0, -1);
  m_freqEditCtl.SetFocus();
  m_freqEditIndex = index;
}

void
CClockGenWnd::OnFreqEditSetFocus()
{
  /* Nothing to do yet */
}

void
CClockGenWnd::OnFreqEditKillFocus()
{
  m_freqEditCtl.ShowWindow(SW_HIDE);
}

LRESULT
CClockGenWnd::OnFreqEditOk(
  WPARAM wParam,
  LPARAM lParam)
{
  UNREFERENCED_PARAMETER(wParam);
  UNREFERENCED_PARAMETER(lParam);

  const DeviceInfo& devInfo = m_pParent->getDeviceInfo();
  ADMXRC3_STATUS status;
  ADMXRC3_HANDLE hDevice = ADMXRC3_HANDLE_INVALID_VALUE;
  CString text;
  double frequency, actualFrequency;

  m_freqEditCtl.GetWindowText(text);
  if (_stscanf_s(text.GetBuffer(), _T("%lf"), &frequency) != 1) {
    return 0;
  }

  status = ADMXRC3_OpenEx(devInfo.m_deviceIndex, FALSE, 0, &hDevice);
  if (ADMXRC3_SUCCESS != status) {
    CString msg;
    msg.Format(_T("Failed to reopen device in active mode\n\nIndex=%u Status=0x%x(%s)\n"), devInfo.m_deviceIndex, status, ADMXRC3_GetStatusString(status, TRUE));
    MessageBox(msg, _T("Failed to set clock generator frequency"), MB_ICONEXCLAMATION | MB_OK);
    return 0;
  }

  status = ADMXRC3_SetClockFrequency(hDevice, m_freqEditIndex, 0, frequency * 1.0e6, &actualFrequency);
  if (ADMXRC3_SUCCESS != status) {
    ADMXRC3_Close(hDevice);
    CString msg;
    msg.Format(_T("Failed to set clock generator %u frequency\nto %.3lf MHz\n\nStatus=0x%x(%s)\n"),
      m_freqEditIndex, frequency, status, ADMXRC3_GetStatusString(status, TRUE));
    MessageBox(msg, _T("Failed to set clock generator frequency"), MB_ICONEXCLAMATION | MB_OK);
    return 0;
  }

  text.Format(_T("%.3lf"), frequency);
  m_listCtl.SetItem((int)m_freqEditIndex + 2, 2, LVIF_TEXT, text.GetBuffer(), 0, 0, 0, (LPARAM)0);

  ADMXRC3_Close(hDevice);

  return 0;
}
