#include "stdafx.h"
#include "afxwinappex.h"

#ifdef _DEBUG
# define new DEBUG_NEW
#endif

#include <commctrl.h>

#include <platform.h>
#include <objmodel.h>

#include "sysmonapp.h"

#pragma comment(linker, "\"/manifestdependency:type='win32' \
                           name='Microsoft.Windows.Common-Controls' \
                           version='6.0.0.0' processorArchitecture='*' \
                           publicKeyToken='6595b64144ccf1df' \
                           language='*'\"")

BEGIN_MESSAGE_MAP(CSysMonApp, CWinApp)
END_MESSAGE_MAP()

class CSysMonCmdLineParser : public CCommandLineInfo {
public:
  CSysMonCmdLineParser(ISysMonWnd* pSysMonWnd) {
    m_pSysMonWnd = pSysMonWnd;
    m_state = ExpectOption;
    m_bError = FALSE;
  }

  void ParseParam(const TCHAR* pszParam, BOOL bFlag, BOOL /* bLast */) {
    unsigned int index;

    switch (m_state) {
    case ExpectOption:
      if (bFlag) {
        if (0 == _tcsicmp(pszParam, _T("verb"))) {
          m_state = ExpectVerb;
        } else {
          /* Unrecognized option; do nothing. */
        }
      } else {
        /* Not an option; do nothing. */
      }
      break;

    case ExpectVerb:
      if (0 == _tcsicmp(pszParam, _T("clr_dev_err"))) {
        if (!m_bError) {
          if (!m_pSysMonWnd->doClearDeviceErrors()) {
            m_bError = TRUE;
          }
        }
        m_state = ExpectOption;
      } else if (0 == _tcsicmp(pszParam, _T("sel_dev_idx"))) {
        m_state = ExpectDeviceIndex;
      } else if (0 == _tcsicmp(pszParam, _T("sel_tab"))) {
        m_state = ExpectTabIndex;
      } else {
        /* Unrecognized verb; return to expecting an option. */
        m_state = ExpectOption;
      }
      break;

    case ExpectDeviceIndex:
      if (1 == _stscanf_s(pszParam, _T("%u"), &index)) {
        if (!m_bError) {
          if (!m_pSysMonWnd->doSelectDeviceIndex(index)) {
            m_bError = TRUE;
          }
        }
      } else {
        /* Not a valid integer; do nothing. */
      }
      m_state = ExpectOption;
      break;

    case ExpectTabIndex:
      if (1 == _stscanf_s(pszParam, _T("%u"), &index)) {
        if (!m_bError) {
          if (!m_pSysMonWnd->doSelectTab(index)) {
            m_bError = TRUE;
          }
        }
      } else {
        /* Not a valid integer; do nothing. */
      }
      m_state = ExpectOption;
      break;

    default: /* Our state has been somehow corrupted. */
      /* Do nothing. */
      break;
    }
  }

protected:
  enum State {
    ExpectOption = 0,
    ExpectVerb,
    ExpectDeviceIndex,
    ExpectTabIndex
  };
  State m_state;
  ISysMonWnd* m_pSysMonWnd;
  BOOL m_bError;
};

static CSysMonApp theApp;

CSysMonApp::CSysMonApp()
{
  m_returnCode = 0;
  m_pMainWnd = NULL;
}

bool_t
CSysMonApp::InitInstance()
{
  // InitCommonControlsEx() is required on Windows XP if an application
  // manifest specifies use of ComCtl32.dll version 6 or later to enable
  // visual styles.  Otherwise, any window creation will fail.
  INITCOMMONCONTROLSEX InitCtrls;
  InitCtrls.dwSize = sizeof(InitCtrls);
  InitCtrls.dwICC = ICC_WIN95_CLASSES | ICC_STANDARD_CLASSES | ICC_BAR_CLASSES | ICC_PROGRESS_CLASS;
  InitCommonControlsEx(&InitCtrls);

  CWinApp::InitInstance();

  m_pSysMonWnd = getFactory().createSysMonWnd();
  if (NULL == m_pSysMonWnd) {
    return FALSE;
  }
  m_pMainWnd = &m_pSysMonWnd->getNativeWindow();

  /* Parse the command line */
  CSysMonCmdLineParser parser(m_pSysMonWnd);
  ParseCommandLine(parser);

  /* Lower our process priority, so that we can run nicely in the background */
  HANDLE hOurProcess = GetCurrentProcess();
  bool_t bSuccess = SetPriorityClass(hOurProcess, BELOW_NORMAL_PRIORITY_CLASS);
  ASSERT(bSuccess);

  return TRUE;
}

int CSysMonApp::ExitInstance()
{
  if (NULL != m_pSysMonWnd) {
    delete m_pSysMonWnd;
    m_pSysMonWnd = NULL;
  }

  CWinApp::ExitInstance();

  return m_returnCode;
}

bool_t
CSysMonApp::OnIdle(
  LONG /* lCount */)
{
  m_pSysMonWnd->idle();

  return 0;
}
